/*****************************************************************************/
/*                      WiNRADiO G35DDC EXCALIBUR ULTRA                      */
/*                                SDK DEMO                                   */
/*                        (C) Copyright RADIXON 2017                         */
/*****************************************************************************/

#include "u_phase.h"
#include <wx/dcbuffer.h>
#include <math.h>
#include "u_global.h"


BEGIN_DECLARE_EVENT_TYPES()
DECLARE_EVENT_TYPE(wxEVT_PHASE_DATACHANGED,5003)
END_DECLARE_EVENT_TYPES()

DEFINE_EVENT_TYPE(wxEVT_PHASE_DATACHANGED)

#define EVT_PHASEDATACHANGED(fn)                                         \
	DECLARE_EVENT_TABLE_ENTRY(wxEVT_PHASE_DATACHANGED, -1, -1,    \
	(wxObjectEventFunction)(wxEventFunction)                        \
	(wxNotifyEventFunction) & fn, (wxObject*) NULL ),

BEGIN_EVENT_TABLE(TPhase, wxPanel)
    EVT_PAINT(TPhase::OnPaintEvent) 
    EVT_SIZE(TPhase::OnSizeEvent)
    EVT_ERASE_BACKGROUND(TPhase::OnEraseBackgroundEvent)
    EVT_PHASEDATACHANGED(TPhase::OnDataChanged)
END_EVENT_TABLE()


TPhase::TPhase(wxWindow *Parent,wxWindowID Id,const wxPoint &Position,const wxSize &Size):
wxPanel(Parent,Id,Position,Size,wxBORDER_SUNKEN|wxFULL_REPAINT_ON_RESIZE),m_Mutex(wxMUTEX_RECURSIVE)
{
    m_YData=NULL;    
    m_Height=GetClientSize().GetHeight();
    m_Width=GetClientSize().GetWidth();
    m_Count=0;
    m_Range=10;
    m_AVG=0;
    m_AVG_From=0;
    m_AVG_To=0;
    m_DataChanged=false;
       
    UpdateParams();
}

TPhase::~TPhase(void)
{
    if(m_YData)
    {
        free(m_YData);
    }
}

void TPhase::UpdateParams(void)
{
 wxClientDC dc(this);
 wxSize Size;
 wxString Str;

    dc.SetFont(GetFont());

    Str.sprintf(wxT("%d"),-m_Range/2);

    Size=dc.GetTextExtent(Str);

    m_GraphRect.SetTopLeft(wxPoint(4+Size.GetWidth()+2,4+Size.GetHeight()/2));
    m_GraphRect.SetBottomRight(wxPoint(GetClientSize().GetWidth()-4,GetClientSize().GetHeight()-4-Size.GetWidth()/2));
    

    if(m_GraphRect.GetWidth()<10)
    {
        m_GraphRect.SetWidth(10);
    }

    if(m_GraphRect.GetHeight()<10)
    {
        m_GraphRect.SetHeight(10);
    }

    m_Width=m_GraphRect.GetWidth();
    m_Height=m_GraphRect.GetHeight();
}

void TPhase::OnSizeEvent(wxSizeEvent &Event)
{
    m_Mutex.Lock();

    Clear();

    UpdateParams();

    m_Mutex.Unlock();    
}

void TPhase::OnEraseBackgroundEvent(wxEraseEvent &Event)
{
    //does nothing
}


void TPhase::Clear(void)
{
    m_Mutex.Lock();
    
    if(m_YData)
    {
        free(m_YData);
        m_YData=NULL;
    }

    m_Count=0;
    m_StartPos=0;

    m_Mutex.Unlock();

    Refresh(false);
}


bool TPhase::Enable(bool enable)
{
    return wxPanel::Enable(enable);
}


void TPhase::Add(float From,float To)
{
 int Index;
 int MinAngle,MaxAngle,Mid;
 float AVG;

    AVG=(From+To)*0.5;

    m_Mutex.Lock();
    
    if(m_YData==NULL)
    {
        m_YData=(int32_pair*)malloc(m_Width*(sizeof(int32_pair)+sizeof(int)));        

        if(m_YData==NULL)
        {
            m_Mutex.Unlock();
            return;
        }

        m_YData_AVG=(int*)(m_YData+m_Width);
        
        m_StartPos=0;
        m_Count=0;
        m_AVG=AVG;
        m_AVG_From=From;
        m_AVG_To=To;
    }
    else
    {
        m_AVG=m_AVG*0.9+AVG*0.1;

        m_AVG_From=m_AVG_From*0.9+From*0.1;
        m_AVG_To=m_AVG_To*0.9+To*0.1;
    }

    MinAngle=-m_Range/2;
    Mid=MaxAngle=m_Range/2;
    
    if(m_Count==m_Width)
    {
        m_StartPos=(m_StartPos+1)%m_Width;
        Index=(m_StartPos+m_Count)%m_Width;
    }
    else
    {
        Index=m_Count++;
    }
    
    m_YData[Index][0]=m_GraphRect.GetBottom()-(From+Mid)*m_Height/m_Range;
    m_YData[Index][1]=m_GraphRect.GetBottom()-(To+Mid)*m_Height/m_Range;

    m_YData_AVG[Index]=m_GraphRect.GetBottom()-(AVG+Mid)*m_Height/m_Range;

    if(!m_DataChanged)
    {
        wxNotifyEvent Event(wxEVT_PHASE_DATACHANGED);

        m_DataChanged=true;

        AddPendingEvent(Event);        
    }

    m_Mutex.Unlock();
}


void TPhase::OnPaintEvent(wxPaintEvent &Event)
{
 wxBufferedPaintDC dc(this);
 wxSize Size;
 wxString Str;
 int y,x;
 uint32_t i,Index1,Index2;
 int32_pair *Y;
 int *Y_AVG;
  
    dc.SetPen(*wxBLACK_PEN);
    dc.SetBrush(*wxBLACK_BRUSH);
    dc.DrawRectangle(wxPoint(0,0),GetClientSize());

    if(IsEnabled())
    {
        dc.SetTextForeground(wxColour(0xFF,0xFF,0xFF));
        dc.SetPen(wxPen(wxColour(0xFF,0xFF,0xFF),1));
    }
    else
    {
        dc.SetTextForeground(wxColour(0x80,0x80,0x80));
        dc.SetPen(wxPen(wxColour(0x80,0x80,0x80),1));
    }

    dc.SetTextBackground(wxColour(0,0,0));
    dc.SetFont(GetFont());

    
    dc.DrawLine(m_GraphRect.GetLeft(),m_GraphRect.GetTop(),m_GraphRect.GetLeft(),m_GraphRect.GetBottom()+1);

    y=(m_GraphRect.GetTop()+m_GraphRect.GetBottom())/2;
    dc.DrawLine(m_GraphRect.GetLeft(),y,m_GraphRect.GetRight()+1,y);

    Str.sprintf(wxT("%d"),m_Range/2);
    Size=dc.GetTextExtent(Str);
    dc.DrawText(Str,m_GraphRect.GetLeft()-2-Size.GetWidth(),4);
    
    Str.sprintf(wxT("0"),m_Range/2);
    Size=dc.GetTextExtent(Str);
    dc.DrawText(Str,m_GraphRect.GetLeft()-2-Size.GetWidth(),y-Size.GetHeight()/2);

    Str.sprintf(wxT("-%d"),m_Range/2);
    Size=dc.GetTextExtent(Str);
    dc.DrawText(Str,m_GraphRect.GetLeft()-2-Size.GetWidth(),m_GraphRect.GetBottom()-Size.GetHeight()/2);

    if(IsEnabled())
    {
        dc.SetPen(wxPen(wxColour(0x00,0xFF,0x00),1));

        m_Mutex.Lock();

        if(m_Count>0)
        {
            dc.SetClippingRegion(m_GraphRect);
            x=m_GraphRect.GetLeft();
            Y=m_YData;

            Index1=m_StartPos;
            Index2=Index1;

            dc.DrawLine(x,Y[Index1][0],x,Y[Index1][1]+1);

            for(i=1;(int)i<m_Count;i++)
            {
                Index1=Index2;
                Index2=(Index2+1)%m_Width;

                x++;
                dc.DrawLine(x,Y[Index1][0],x,Y[Index1][1]+1);

                if(Y[Index2][0]>Y[Index1][1])
                {
                    dc.DrawLine(x-1,Y[Index1][1],x,Y[Index2][0]);
                }
                else if(Y[Index2][1]<Y[Index1][0])
                {
                    dc.DrawLine(x-1,Y[Index1][0],x,Y[Index2][1]);
                }
            }

            dc.SetPen(wxPen(wxColour(0xFF,0xFF,0x00),1));

            Index1=m_StartPos;
            Index2=Index1;
            x=m_GraphRect.GetLeft()+1;

            Y_AVG=m_YData_AVG;

            for(i=1;(int)i<m_Count;i++)
            {

                Index1=Index2;
                Index2=(Index2+1)%m_Width;

                dc.DrawLine(x-1,Y_AVG[Index1],x,Y_AVG[Index2]);

                x++;
            }
        }

        Str.sprintf(wxT("AVG: %6.2f  [Min: %6.2f / Max: %6.2f]"),m_AVG,m_AVG_From,m_AVG_To); 

        m_Mutex.Unlock();
    
        dc.SetTextForeground(wxColour(0xFF,0x00,0x00));

        Size=dc.GetTextExtent(Str);

        dc.DrawText(Str,m_GraphRect.GetRight()-Size.GetWidth(),m_GraphRect.GetBottom()-Size.GetHeight());
    }
}

void TPhase::SetRange(int Range)
{
   if(m_Range!=Range && Range>0)
   {
       m_Mutex.Lock();
       m_Range=Range;
       Clear();
       UpdateParams();
       m_Mutex.Unlock();
       Refresh(false);
   }
}

void TPhase::OnDataChanged(wxNotifyEvent &Event)
{
    m_Mutex.Lock();
    m_DataChanged=false;
    m_Mutex.Unlock();

    Refresh(false);
}