/*****************************************************************************/
/*                      WiNRADiO G35DDC EXCALIBUR ULTRA                      */
/*                                SDK DEMO                                   */
/*                      Simple frequency spectrum panel                      */
/*                        (C) Copyright RADIXON 2017                         */
/*****************************************************************************/

#include "u_spectrum.h"
#include <wx/dcbuffer.h>
#include <math.h>
#include "u_global.h"

IMPLEMENT_DYNAMIC_CLASS(TSpectrumTuneEvent,wxEvent)

BEGIN_DECLARE_EVENT_TYPES()
DECLARE_EVENT_TYPE(wxEVT_COMMAND_DATACHANGED,5002)
END_DECLARE_EVENT_TYPES()

DEFINE_EVENT_TYPE(wxEVT_COMMAND_DATACHANGED)
DEFINE_EVENT_TYPE(wxEVT_COMMAND_SPECTRUM_TUNE)

#define EVT_DATACHANGED(fn)                                         \
	DECLARE_EVENT_TABLE_ENTRY(wxEVT_COMMAND_DATACHANGED, -1, -1,    \
	(wxObjectEventFunction)(wxEventFunction)                        \
	(wxNotifyEventFunction) & fn, (wxObject*) NULL ),


BEGIN_EVENT_TABLE(TSpectrum, wxPanel)
    EVT_PAINT(TSpectrum::OnPaintEvent)
    EVT_SIZE(TSpectrum::OnSizeEvent)
    EVT_ERASE_BACKGROUND(TSpectrum::OnEraseBackgroundEvent)
    EVT_DATACHANGED(TSpectrum::OnData)
    EVT_LEFT_DOWN(TSpectrum::OnMouseDown)
    EVT_LEFT_UP(TSpectrum::OnMouseUp)
    EVT_MOTION(TSpectrum::OnMouseMove)
    EVT_MOUSE_CAPTURE_LOST(TSpectrum::OnMouseCaptureLost)
END_EVENT_TABLE()

TSpectrumTuneEvent::TSpectrumTuneEvent(int Id,int64_t Frequency):wxNotifyEvent(wxEVT_COMMAND_SPECTRUM_TUNE,Id)
{
    m_Frequency=Frequency;
}

TSpectrumTuneEvent::TSpectrumTuneEvent(const TSpectrumTuneEvent &Event):wxNotifyEvent(Event)
{
    m_Frequency=Event.m_Frequency;
}

wxEvent* TSpectrumTuneEvent::Clone(void) const
{
    return new TSpectrumTuneEvent(*this);
}

TSpectrum::TSpectrum(wxWindow *Parent,wxWindowID Id,const wxPoint &Position,const wxSize &Size):
wxPanel(Parent,Id,Position,Size,wxBORDER_SUNKEN|wxFULL_REPAINT_ON_RESIZE),m_Mutex(wxMUTEX_RECURSIVE)
{
    m_Range=20000;
    m_CenterFrequency=15000000;
    m_MinLevel=-140;
    m_Data=NULL;
    m_DataCount=0;
    m_DataChanged=false;
    m_DataValid=false;

    m_DisplayDataCount=0;
    m_DisplayData=NULL;
    m_DisplayDataValid=false;
    m_BandFrequencyCursor=true;
    m_TuneWhenMouseMove=false;

    m_Band=SpectrumBand_BothSides;
    m_BandFrequency=15000000;
    m_BandWidth=10000;
    m_BandRelative=false;

    UpdateParams();
}

TSpectrum::~TSpectrum(void)
{
    if(m_Data)
    {
        delete[] m_Data;
    }

    if(m_DisplayData)
    {
        delete[] m_DisplayData;
    }
}

void TSpectrum::UpdateParams(void)
{
 wxClientDC dc(this);
 wxSize Size;
 double Step;
 int Dec;
 double Num;

    dc.SetFont(GetFont());
    Size=dc.GetTextExtent(wxT("-999"));

    m_GraphRect.SetTopLeft(wxPoint(Size.GetWidth()+2+2,Size.GetHeight()/2+2));
    m_GraphRect.SetBottomRight(wxPoint(GetClientSize().GetWidth()-5,GetClientSize().GetHeight()-Size.GetHeight()-2-2));

    if(m_GraphRect.GetWidth()<10)
    {
        m_GraphRect.SetWidth(10);
    }

    if(m_GraphRect.GetHeight()<10)
    {
        m_GraphRect.SetHeight(10);
    }

    m_Frequency2Pixel=(double)(m_GraphRect.GetWidth()-2)/m_Range;
    m_Pixel2Frequency=(double)m_Range/(m_GraphRect.GetWidth()-2);

    //calculate freq grid steps
    Step=m_Pixel2Frequency*95;

    Dec=(int)pow(10.0,(int)log10(Step));
    Num=(double)Step/(double)Dec;

    if(Num>5.0)
    {
        m_FrequencyStep=10;
    }
    else if(Num>2.0)
    {
        m_FrequencyStep=5;
    }
    else
    {
        m_FrequencyStep=2;
    }

    m_FrequencyStep*=Dec;

    m_Level2Pixel=(double)(m_GraphRect.GetHeight()-1)/-m_MinLevel;
    m_Pixel2Level=1.0/m_Level2Pixel;

    m_LevelStep=5;

    while(m_LevelStep*m_Level2Pixel<=(Size.GetHeight()))
    {
        if(m_LevelStep==5)
        {
            m_LevelStep=10;
        }
        else
        {
            m_LevelStep+=10;
        }
    }
}

void TSpectrum::OnSizeEvent(wxSizeEvent &Event)
{
    UpdateParams();
    UpdateDisplayData();
}

void TSpectrum::OnEraseBackgroundEvent(wxEraseEvent &Event)
{
    //does nothing
}

void TSpectrum::OnPaintEvent(wxPaintEvent &Event)
{
 wxBufferedPaintDC dc(this);
 int64_t Frequency;
 int64_t StartFrequency;
 double fx,fy;
 double Step;
 int x,y;
 wxString s;
 wxSize Size;
 int Level;
 int Band_x1,Band_x2;
 int Band_x;
 bool BandFrequencyCursor;
 int64_t BandFrequency;

    dc.SetPen(*wxBLACK_PEN);
    dc.SetBrush(*wxBLACK_BRUSH);
    dc.DrawRectangle(wxPoint(0,0),GetClientSize());

    BandFrequencyCursor=false;

    if(IsEnabled() && m_Band)
    {
        BandFrequency=m_BandFrequency;

        if(!m_BandRelative)
        {
            BandFrequency-=m_CenterFrequency;
        }

        switch(m_Band)
        {
            case SpectrumBand_BothSides:
                Band_x1=(BandFrequency+m_Range*0.5-m_BandWidth*0.5)*m_Frequency2Pixel+0.5;
                Band_x2=(BandFrequency+m_Range*0.5+m_BandWidth*0.5)*m_Frequency2Pixel+0.5;
                Band_x=(BandFrequency+m_Range*0.5)*m_Frequency2Pixel+0.5;
                break;

            case SpectrumBand_LowerSide:
                Band_x1=(BandFrequency+m_Range*0.5-m_BandWidth)*m_Frequency2Pixel+0.5;
                Band_x2=(BandFrequency+m_Range*0.5)*m_Frequency2Pixel+0.5;
                Band_x=Band_x2;
                break;

            case SpectrumBand_UpperSide:
                Band_x1=(BandFrequency+m_Range*0.5)*m_Frequency2Pixel+0.5;
                Band_x2=(BandFrequency+m_Range*0.5+m_BandWidth)*m_Frequency2Pixel+0.5;
                Band_x=Band_x1;
                break;

            default:
                break;
        }

        if(Band_x2>=0 && Band_x1<m_GraphRect.GetWidth())
        {
            if(Band_x1<0)
            {
                Band_x1=0;
            }
            else if(Band_x2>=m_GraphRect.GetWidth())
            {
                Band_x2=m_GraphRect.GetWidth()-1;
            }

            Band_x1+=m_GraphRect.GetLeft();
            Band_x2+=m_GraphRect.GetLeft();

            dc.SetPen(wxPen(wxColour(0,0x38,0)));
            dc.SetBrush(wxBrush(wxColour(0,0x38,0)));
            dc.DrawRectangle(Band_x1,m_GraphRect.GetTop(),Band_x2-Band_x1+1,m_GraphRect.GetHeight());

            if(m_BandFrequencyCursor && Band_x>=0 && Band_x<m_GraphRect.GetWidth())
            {
                Band_x+=m_GraphRect.GetLeft();

                BandFrequencyCursor=true;
            }
        }
    }

    if(IsEnabled())
    {
        dc.SetTextForeground(wxColour(0xFF,0xFF,0xFF));
        dc.SetPen(wxPen(wxColour(0xFF,0xFF,0xFF),1));
    }
    else
    {
        dc.SetTextForeground(wxColour(0x80,0x80,0x80));
        dc.SetPen(wxPen(wxColour(0x80,0x80,0x80),1));
    }

    dc.SetTextBackground(wxColour(0,0,0));
    dc.SetFont(GetFont());

    StartFrequency=m_CenterFrequency-(m_Range>>1);
    Frequency=(int64_t)(StartFrequency/m_FrequencyStep)*m_FrequencyStep;
    fx=(Frequency-StartFrequency)*m_Frequency2Pixel+m_GraphRect.GetLeft();
    Step=m_FrequencyStep*m_Frequency2Pixel;

    for(;;)
    {
        x=(int)(fx+0.5);

        if(x>m_GraphRect.GetRight())
        {
            break;
        }

        if(x>=m_GraphRect.GetLeft())
        {
            dc.DrawLine(x,m_GraphRect.GetBottom(),x,m_GraphRect.GetBottom()+2);

            s=FormatFrequency(Frequency);

            Size=dc.GetTextExtent(s);

            x-=Size.GetWidth()>>1;

            if(x<m_GraphRect.GetLeft())
            {
                x=m_GraphRect.GetLeft();
            }
            else if(x+Size.GetWidth()>m_GraphRect.GetRight())
            {
                x=m_GraphRect.GetRight()-Size.GetWidth();
            }

            dc.DrawText(s,x,m_GraphRect.GetBottom()+2);
        }

        fx+=Step;
        Frequency+=m_FrequencyStep;
    }

    //level
    fy=m_GraphRect.GetTop();
    Step=m_LevelStep*m_Level2Pixel;
    Level=0;

    for(;;)
    {
        y=(int)(fy+0.5);

        if(y>m_GraphRect.GetBottom())
        {
            break;
        }

        dc.DrawLine(m_GraphRect.GetLeft(),y,m_GraphRect.GetLeft()-2,y);

        s.sprintf(wxT("%d"),Level);

        Size=dc.GetTextExtent(s);

        dc.DrawText(s,m_GraphRect.GetLeft()-2-Size.GetWidth(),y-Size.GetHeight()/2);

        fy+=Step;
        Level-=m_LevelStep;
    }

    if(IsEnabled())
    {
        dc.SetPen(wxPen(wxColour(0xE0,0xE0,0xE0),1,wxSOLID));
    }
    else
    {
        dc.SetPen(wxPen(wxColour(0x70,0x70,0x70),1,wxSOLID));
    }

    dc.DrawLine(m_GraphRect.GetTopLeft(),m_GraphRect.GetBottomLeft());
    dc.DrawLine(m_GraphRect.GetBottomLeft(),m_GraphRect.GetBottomRight());

    if(BandFrequencyCursor)
    {
        dc.SetPen(wxPen(wxColour(0xFF,0,0),1));
        dc.DrawLine(Band_x,m_GraphRect.GetTop(),Band_x,m_GraphRect.GetBottom());
    }

    if(m_DisplayDataValid && IsEnabled())
    {
        dc.SetClippingRegion(m_GraphRect);
        dc.SetPen(wxPen(wxColour(0x30,0xE0,0x0),1,wxSOLID));
        dc.DrawLines(m_DisplayDataCount,m_DisplayData);
    }

    if(m_Text.Length())
    {
        if(IsEnabled())
        {
            dc.SetTextForeground(wxColour(0xFF,0x60,0x30));
        }
        else
        {
            dc.SetTextForeground(wxColour(0x80,0x30,0x20));
        }

        dc.SetTextBackground(wxColour(0,0,0,0));


        Size=dc.GetTextExtent(m_Text);

        dc.DrawText(m_Text,m_GraphRect.GetRight()-Size.GetWidth(),m_GraphRect.GetTop());
    }
}

void TSpectrum::SetData(float *Data,uint32_t Count,double Range,double Offset)
{
    if(Count==0)
    {
        return;
    }

    m_Mutex.Lock();

    if(m_DataCount!=Count)
    {
        if(m_Data)
        {
            delete[] m_Data;
        }

        m_Data=new float[Count];
        m_DataCount=Count;
    }

    memcpy(m_Data,Data,Count*sizeof(float));
    m_DataRange=Range;
    m_DataOffset=Offset;

    m_DataValid=true;

    if(!m_DataChanged)
    {
        wxNotifyEvent Event(wxEVT_COMMAND_DATACHANGED);

        m_DataChanged=true;

        AddPendingEvent(Event);
    }

    m_Mutex.Unlock();
}

void TSpectrum::OnData(wxNotifyEvent &Event)
{
    m_Mutex.Lock();

    m_DataChanged=false;

    UpdateDisplayData();

    m_Mutex.Unlock();

    Refresh(false);
}

void TSpectrum::UpdateDisplayData(void)
{
 uint32_t i;
 int Offset;
 double k;

    m_Mutex.Lock();

    if(m_DataValid)
    {
        k=(double)(m_DataCount+1)/m_DataRange;

        if(m_DisplayDataCount!=m_DataCount)
        {
            if(m_DisplayData)
            {
                delete[] m_DisplayData;
            }

            m_DisplayData=new wxPoint[m_DataCount];
            m_DisplayDataCount=m_DataCount;
        }

        Offset=-m_DataOffset*m_Frequency2Pixel;
        k=(1/k)*m_Frequency2Pixel;

        for(i=0;i<m_DataCount;i++)
        {
            m_DisplayData[i].x=m_GraphRect.GetLeft()+Offset+i*k;
            m_DisplayData[i].y=m_GraphRect.GetBottom()-(m_Data[i]-m_MinLevel)*m_Level2Pixel;
        }

        m_DisplayDataValid=true;
    }

    m_Mutex.Unlock();
}

void TSpectrum::SetCenterFrequency(int64_t Frequency)
{
    if(m_CenterFrequency!=Frequency)
    {
        m_CenterFrequency=Frequency;
        Refresh(false);
    }
}

void TSpectrum::SetRange(int Range)
{
    if(m_Range!=Range)
    {
        m_Range=Range;
        UpdateParams();
        UpdateDisplayData();
        Refresh(false);
    }
}

bool TSpectrum::Enable(bool enable)
{
    if(!enable)
    {
        m_TuneWhenMouseMove=false;
    }

    enable=wxPanel::Enable(enable);

    Refresh(false);

    return enable;
}

void TSpectrum::SetBand(TSpectrumBand Band,bool Relative)
{
    if(m_Band!=Band || m_BandRelative!=Relative)
    {
        m_Band=Band;
        m_BandRelative=Relative;

        Refresh(false);
    }
}

void TSpectrum::Clear(void)
{
    m_Mutex.Lock();
    m_DataValid=false;
    m_DataChanged=false;
    m_Mutex.Unlock();

    m_DisplayDataValid=false;
    Refresh(false);
}

void TSpectrum::SetMinLevel(int Level)
{
    if(Level>-10)
    {
        Level=-10;
    }

    if(m_MinLevel!=Level)
    {
        m_MinLevel=Level;

        UpdateParams();
        UpdateDisplayData();

        Refresh(false);
    }
}

void TSpectrum::SetBandFrequency(int64_t Frequency)
{
    if(m_BandFrequency!=Frequency)
    {
        m_BandFrequency=Frequency;

        Refresh(false);
    }
}

void TSpectrum::SetBandWidth(uint32_t Width)
{
    if(m_BandWidth!=(int)Width)
    {
        m_BandWidth=Width;

        Refresh(false);
    }
}

void TSpectrum::OnMouseDown(wxMouseEvent &Event)
{
    m_TuneWhenMouseMove=true;

    CaptureMouse();

    Tune(Event.GetPosition());

    Event.Skip();
}

void TSpectrum::OnMouseUp(wxMouseEvent &Event)
{
    if(m_TuneWhenMouseMove)
    {
        m_TuneWhenMouseMove=false;
        ReleaseMouse();
    }

    Event.Skip();
}

void TSpectrum::OnMouseMove(wxMouseEvent &Event)
{
    if(m_TuneWhenMouseMove)
    {
        Tune(Event.GetPosition());
    }

    Event.Skip();
}

void TSpectrum::Tune(const wxPoint &Pos)
{
 int64_t Frequency;
 int Min,Max;

    if(m_GraphRect.Contains(Pos))
    {
        Frequency=((Pos.x-m_GraphRect.GetLeft())*m_Pixel2Frequency+0.5)-m_Range/2;

        switch(m_Band)
        {
            case SpectrumBand_None:
                return;

            case SpectrumBand_BothSides:
                Max=(m_Range-m_BandWidth)/2;
                Min=-Max;
                break;

            case SpectrumBand_LowerSide:
                Max=m_Range/2;
                Min=-(m_Range/2-m_BandWidth);
                break;

            case SpectrumBand_UpperSide:
                Min=-m_Range/2;
                Max=m_Range/2-m_BandWidth;
                break;
        }

        if(Frequency<Min)
        {
            Frequency=Min;
        }
        else if(Frequency>Max)
        {
            Frequency=Max;
        }

        if(!m_BandRelative)
        {
            Frequency+=m_CenterFrequency;
        }

        TSpectrumTuneEvent Event(GetId(),Frequency);

        ProcessEvent(Event);
    }
}

void TSpectrum::OnMouseCaptureLost(wxMouseCaptureLostEvent &Event)
{
    m_TuneWhenMouseMove=false;
}

void TSpectrum::SetText(const wxString &Text)
{
    m_Text=Text;
    Refresh(false);
}
