/* ***********************************************************************/
/*                                                                       */
/*                       WiNRADiO G35DDC Excalibur                       */
/*               C/C++ header file for G35DDC receiver API               */
/*           (C) Copyright 2017 RADIXON. All rights reserved.            */
/*                                                                       */
/* ***********************************************************************/

#ifndef __G35DDCAPI_H__
#define __G35DDCAPI_H__

#include <stdint.h>

#pragma pack(push,1)

#define G35DDC_CLASS_ID_DEVICE_ENUMERATOR           0x35001
#define G35DDC_CLASS_ID_DEVICE                      0x35102
#define G35DDC_CLASS_ID_COH_DEVICE_SET_ENUMERATOR   0x35003
#define G35DDC_CLASS_ID_COH_DEVICE_SET              0x35104

#ifndef __G3XDDC_INTERFACE__
#define __G3XDDC_INTERFACE__ 5
#define G3XDDC_INTERFACE_TYPE_PCIE          0
#define G3XDDC_INTERFACE_TYPE_USB           1
#endif //__G3XDDC_INTERFACE__

#ifndef __G3XDDC_OPEN__
#define __G3XDDC_OPEN__ 5
#define G3XDDC_OPEN_DEMO   ((char*)0)
#define G3XDDC_OPEN_FIRST  ((char*)-1l)
#endif //__G3XDDC_OPEN__

#if !defined(__G3XDDC_MODES__) || __G3XDDC_MODES__<1
#define G3XDDC_MODE_CW               0
#define G3XDDC_MODE_AM               1
#define G3XDDC_MODE_FM               2
#define G3XDDC_MODE_LSB              4
#define G3XDDC_MODE_USB              5
#define G3XDDC_MODE_AMS              8
#endif //__G3XDDC_MODES__

#if !defined(__G3XDDC_MODES__) || __G3XDDC_MODES__<3
#undef __G3XDDC_MODES__
#define __G3XDDC_MODES__ 5

#define G3XDDC_MODE_DSB              13
#define G3XDDC_MODE_ISB              14
#endif //__G3XDDC_MODES__

#define G35DDC_MODE_COUNT            8

#ifndef __G3XDDC_SIDE_BAND__
#define __G3XDDC_SIDE_BAND__ 5
#define G3XDDC_SIDE_BAND_LOWER         0x01
#define G3XDDC_SIDE_BAND_UPPER         0x02
#define G3XDDC_SIDE_BAND_BOTH          (G3XDDC_SIDE_BAND_LOWER|G3XDDC_SIDE_BAND_UPPER)
#endif //__G3XDDC_SIDE_BAND__


#if !defined(__G3XDDC_DEMODULATOR_PARAMS__) || __G3XDDC_DEMODULATOR_PARAMS__<1
#define G3XDDC_DEMODULATOR_PARAM_AMS_SIDE_BAND          0x00000001
#define G3XDDC_DEMODULATOR_PARAM_AMS_CAPTURE_RANGE      0x00000002
#define G3XDDC_DEMODULATOR_PARAM_CW_FREQUENCY           0x00000003

typedef struct
{
    uint32_t  Tune;   //Hz
    uint32_t  Lock;   //Hz
} G3XDDC_AMS_CAPTURE_RANGE;
#endif //__G3XDDC_DEMODULATOR_PARAMS__

#if !defined(__G3XDDC_DEMODULATOR_PARAMS__) || __G3XDDC_DEMODULATOR_PARAMS__<3
#undef __G3XDDC_DEMODULATOR_PARAMS__
#define __G3XDDC_DEMODULATOR_PARAMS__ 5
#define G3XDDC_DEMODULATOR_PARAM_DSB_SIDE_BAND          0x00000006
#define G3XDDC_DEMODULATOR_PARAM_ISB_SIDE_BAND          0x00000007
#endif //__G3XDDC_DEMODULATOR_PARAMS__

#if !defined(__G3XDDC_DEMODULATOR_STATES__) || __G3XDDC_DEMODULATOR_STATES__<1

#define G3XDDC_DEMODULATOR_STATE_AMS_LOCK               0x00000001
#define G3XDDC_DEMODULATOR_STATE_AMS_FREQUENCY          0x00000002
#define G3XDDC_DEMODULATOR_STATE_AM_DEPTH               0x00000003
#define G3XDDC_DEMODULATOR_STATE_TUNE_ERROR             0x00000004
#define G3XDDC_DEMODULATOR_STATE_FM_DEVIATION           0x00000006

#endif //__G3XDDC_DEMODULATOR_STATES__

#if !defined(__G3XDDC_DEMODULATOR_STATES__) || __G3XDDC_DEMODULATOR_STATES__<3

#undef __G3XDDC_DEMODULATOR_STATES__
#define __G3XDDC_DEMODULATOR_STATES__ 5

#define G3XDDC_DEMODULATOR_STATE_DSB_LOCK               0x00000007
#define G3XDDC_DEMODULATOR_STATE_DSB_FREQUENCY          0x00000008

#endif //__G3XDDC_DEMODULATOR_STATES__

#define G35_AUDIO_SAMPLE_RATE                           48000

#ifndef __G3XDDC_DEVICE_STATE__
#define __G3XDDC_DEVICE_STATE__ 5

#define G3XDDC_DEVICE_STATE_ERROR_HIGH_TEMP             0x00000001

#endif //__G3XDDC_DEVICE_STATE__

typedef int (*G35DDCAPI_CREATE_INSTANCE)(uint32_t Code,void **Intf);

#define G35DDC_FLAGS_EXTERNAL_REFERENCE     0x00000001
#define G35DDC_FLAGS_AUDIO_OUTPUT			0x00000002
#define G35DDC_FLAGS_COHERENT			    0x00000004
#define G35DDC_FLAGS_BIT                    0x00000010

typedef struct
{
    char        DevicePath[64];
    uint8_t     InterfaceType;
    char        SerialNumber[9];
    uint16_t    HWVersion;
    uint16_t    FWVersion;
    uint8_t     EEPROMVersion;
    uint32_t    Flags;
    uint32_t    ChannelCount;
    uint32_t    DDCTypeCount;
} G35DDC_DEVICE_INFO;

#define G35DDC_OPEN_DEMO_SET    ((G35DDC_DEVICE_INFO*)0x00000000)
#define G35DDC_OPEN_FIRST_SET   ((G35DDC_DEVICE_INFO*)0xFFFFFFFF)

#ifndef __G3XDDC_DDC_INFO__
#define __G3XDDC_DDC_INFO__ 5

typedef struct
{
    uint32_t      SampleRate;
    uint32_t      Bandwidth;
    uint32_t      BitsPerSample;
} G3XDDC_DDC_INFO;

#endif //__G3XDDC_DDC_INFO__

#ifdef __cplusplus

#define G35DDC_DECLARE_INTERFACE(__name__) struct __name__
#define G35DDC_THIS
#define G35DDC_METHOD(__type__,__method__,__params__) virtual __type__ __method__ __params__=0
#define G35DDC_METHOD_VOID(__type__,__method__) virtual __type__ __method__(void)=0
#else //__cplusplus
#define G35DDC_THIS struct G35DDC_INTERFACE *This,
#define G35DDC_DECLARE_INTERFACE(__name__) typedef struct __name__ { const struct __name__##Vtbl *lpVtbl;} __name__;\
    typedef const struct __name__##Vtbl __name__##Vtbl;struct __name__##Vtbl
#define G35DDC_METHOD(__type__,__method__,__params__) __type__ (*__method__)__params__
#define G35DDC_METHOD_VOID(__type__,__method__) __type__ (*__method__)(struct G35DDC_INTERFACE *This)
#endif //__cplusplus

#undef G35DDC_INTERFACE
#define G35DDC_INTERFACE IG35DDCDeviceEnumerator

G35DDC_DECLARE_INTERFACE(IG35DDCDeviceEnumerator)
{
    G35DDC_METHOD_VOID(uint32_t,AddRef);
    G35DDC_METHOD_VOID(uint32_t,Release);

    G35DDC_METHOD_VOID(int,Enumerate);
    G35DDC_METHOD_VOID(uint32_t,GetCount);
    G35DDC_METHOD(int,GetDeviceInfo,(G35DDC_THIS uint32_t DeviceIndex,G35DDC_DEVICE_INFO *DeviceInfo,uint32_t BufferLength));
};

#undef G35DDC_INTERFACE
#define G35DDC_INTERFACE IG35DDCDeviceCallback

struct IG35DDCDevice;

G35DDC_DECLARE_INTERFACE(IG35DDCDeviceCallback)
{
    G35DDC_METHOD(void,G35DDC_IFCallback,(G35DDC_THIS struct IG35DDCDevice *Device,const int16_t *Buffer,uint32_t NumberOfSamples,uint16_t MaxADCAmplitude,uint32_t ADCSampleRate));
    G35DDC_METHOD(void,G35DDC_DDC1StreamCallback,(G35DDC_THIS struct IG35DDCDevice *Device,const void *Buffer,uint32_t NumberOfSamples,uint32_t BitsPerSample));
    G35DDC_METHOD(int,G35DDC_DDC1PlaybackStreamCallback,(G35DDC_THIS struct IG35DDCDevice *Device,void *Buffer,uint32_t NumberOfSamples,uint32_t BitsPerSample));
    G35DDC_METHOD(void,G35DDC_DDC2StreamCallback,(G35DDC_THIS struct IG35DDCDevice *Device,uint32_t Channel,const float *Buffer,uint32_t NumberOfSamples));
    G35DDC_METHOD(void,G35DDC_DDC2PreprocessedStreamCallback,(G35DDC_THIS struct IG35DDCDevice *Device,uint32_t Channel,const float *Buffer,uint32_t NumberOfSamples,float SlevelPeak,float SlevelRMS));
    G35DDC_METHOD(void,G35DDC_AudioStreamCallback,(G35DDC_THIS struct IG35DDCDevice *Device,uint32_t Channel,uint32_t Type,const float *Buffer,uint32_t NumberOfSamples));
    G35DDC_METHOD(int,G35DDC_AudioPlaybackStreamCallback,(G35DDC_THIS struct IG35DDCDevice *Device,uint32_t Channel,float *Buffer,uint32_t NumberOfSamples));
};

#undef G35DDC_INTERFACE
#define G35DDC_INTERFACE IG35DDCDevice

G35DDC_DECLARE_INTERFACE(IG35DDCDevice)
{
    G35DDC_METHOD_VOID(uint32_t,AddRef);
    G35DDC_METHOD_VOID(uint32_t,Release);

    G35DDC_METHOD(int,Open,(G35DDC_THIS const char *DevicePath));
    G35DDC_METHOD_VOID(void,Close);

    G35DDC_METHOD_VOID(int,IsOpen);
    G35DDC_METHOD_VOID(int,IsConnected);

    G35DDC_METHOD(void,SetCallback,(G35DDC_THIS IG35DDCDeviceCallback *Callback));
    G35DDC_METHOD_VOID(IG35DDCDeviceCallback*,GetCallback);

    G35DDC_METHOD(int,GetDeviceInfo,(G35DDC_THIS G35DDC_DEVICE_INFO *Info,uint32_t BufferLength));

    G35DDC_METHOD(int,SetPower,(G35DDC_THIS int Power));
    G35DDC_METHOD(int,GetPower,(G35DDC_THIS int *Power));

    G35DDC_METHOD(int,SetExternalReference,(G35DDC_THIS int Enabled));
    G35DDC_METHOD(int,GetExternalReference,(G35DDC_THIS int *Enabled));

    G35DDC_METHOD(int,SetAttenuator,(G35DDC_THIS uint32_t Attenuator));
    G35DDC_METHOD(int,GetAttenuator,(G35DDC_THIS uint32_t *Attenuator));
    G35DDC_METHOD(int,SetPreselectors,(G35DDC_THIS uint32_t Low,uint32_t High));
    G35DDC_METHOD(int,GetPreselectors,(G35DDC_THIS uint32_t *Low,uint32_t *High));
    G35DDC_METHOD(int,SetNoiseGenerator,(G35DDC_THIS int Enabled));
    G35DDC_METHOD(int,GetNoiseGenerator,(G35DDC_THIS int *Enabled));
    G35DDC_METHOD(int,SetPreamplifier,(G35DDC_THIS int Preamp));
    G35DDC_METHOD(int,GetPreamplifier,(G35DDC_THIS int *Preamp));

    G35DDC_METHOD(int,SetInverted,(G35DDC_THIS int Inverted));
    G35DDC_METHOD(int,GetInverted,(G35DDC_THIS int *Inverted));

    G35DDC_METHOD(int,SetADCNoiseBlanker,(G35DDC_THIS int Enabled));
    G35DDC_METHOD(int,GetADCNoiseBlanker,(G35DDC_THIS int *Enabled));
    G35DDC_METHOD(int,SetADCNoiseBlankerThreshold,(G35DDC_THIS uint16_t Threshold));
    G35DDC_METHOD(int,GetADCNoiseBlankerThreshold,(G35DDC_THIS uint16_t *Threshold));

    G35DDC_METHOD(int,StartIF,(G35DDC_THIS uint16_t Period));
    G35DDC_METHOD_VOID(int,StopIF);

    G35DDC_METHOD(int,GetDDCInfo,(G35DDC_THIS uint32_t DDCIndex,G3XDDC_DDC_INFO *Info));
    G35DDC_METHOD(int,GetDDC1Count,(G35DDC_THIS uint32_t *Count));
    G35DDC_METHOD(int,SetDDC1,(G35DDC_THIS uint32_t DDCIndex));
    G35DDC_METHOD(int,GetDDC1,(G35DDC_THIS uint32_t *DDCTypeIndex, G3XDDC_DDC_INFO *DDCInfo));

    G35DDC_METHOD(int,StartDDC1,(G35DDC_THIS uint32_t SamplesPerBuffer));
    G35DDC_METHOD(int,StopDDC1,());

    G35DDC_METHOD(int,StartDDC1Playback,(G35DDC_THIS uint32_t SamplesPerBuffer,uint32_t BitsPerSample));
    G35DDC_METHOD(int,PauseDDC1Playback,());
    G35DDC_METHOD(int,ResumeDDC1Playback,());

    G35DDC_METHOD(int,SetDDC1Frequency,(G35DDC_THIS uint32_t Frequency));
    G35DDC_METHOD(int,GetDDC1Frequency,(G35DDC_THIS uint32_t *Frequency));

    G35DDC_METHOD(int,GetDDC2,(G35DDC_THIS uint32_t *DDCTypeIndex,G3XDDC_DDC_INFO *DDCInfo));

    G35DDC_METHOD(int,StartDDC2,(G35DDC_THIS uint32_t Channel,uint32_t SamplesPerBuffer));
    G35DDC_METHOD(int,StopDDC2,(G35DDC_THIS uint32_t Channel));

    G35DDC_METHOD(int,SetDDC2Frequency,(G35DDC_THIS uint32_t Channel,int32_t Frequency));
    G35DDC_METHOD(int,GetDDC2Frequency,(G35DDC_THIS uint32_t Channel,int32_t *Frequency));

    G35DDC_METHOD(int,SetDDC2NoiseBlanker,(G35DDC_THIS uint32_t Channel,int Enabled));
    G35DDC_METHOD(int,GetDDC2NoiseBlanker,(G35DDC_THIS uint32_t Channel,int *Enabled));
    G35DDC_METHOD(int,SetDDC2NoiseBlankerThreshold,(G35DDC_THIS uint32_t Channel,double Threshold));
    G35DDC_METHOD(int,GetDDC2NoiseBlankerThreshold,(G35DDC_THIS uint32_t Channel,double *Threshold));
    G35DDC_METHOD(int,GetDDC2NoiseBlankerExcessValue,(G35DDC_THIS uint32_t Channel,double *Value));

    G35DDC_METHOD(int,GetSignalLevel,(G35DDC_THIS uint32_t Channel,float *Peak,float *RMS));

    G35DDC_METHOD(int,SetAGC,(G35DDC_THIS uint32_t Channel,int Enabled));
    G35DDC_METHOD(int,GetAGC,(G35DDC_THIS uint32_t Channel,int *Enabled));
    G35DDC_METHOD(int,SetAGCParams,(G35DDC_THIS uint32_t Channel,double AttackTime,double DecayTime,double ReferenceLevel));
    G35DDC_METHOD(int,GetAGCParams,(G35DDC_THIS uint32_t Channel,double *AttackTime,double *DecayTime,double *ReferenceLevel));
    G35DDC_METHOD(int,SetMaxAGCGain,(G35DDC_THIS uint32_t Channel,double MaxGain));
    G35DDC_METHOD(int,GetMaxAGCGain,(G35DDC_THIS uint32_t Channel,double *MaxGain));
    G35DDC_METHOD(int,SetGain,(G35DDC_THIS uint32_t Channel,double Gain));
    G35DDC_METHOD(int,GetGain,(G35DDC_THIS uint32_t Channel,double *Gain));
    G35DDC_METHOD(int,GetCurrentGain,(G35DDC_THIS uint32_t Channel,double *Gain));

    G35DDC_METHOD(int,SetNotchFilter,(G35DDC_THIS uint32_t Channel,uint32_t FilterIndex,int Enabled));
    G35DDC_METHOD(int,GetNotchFilter,(G35DDC_THIS uint32_t Channel,uint32_t FilterIndex,int *Enabled));
    G35DDC_METHOD(int,SetNotchFilterFrequency,(G35DDC_THIS uint32_t Channel,uint32_t FilterIndex,int32_t Frequency));
    G35DDC_METHOD(int,GetNotchFilterFrequency,(G35DDC_THIS uint32_t Channel,uint32_t FilterIndex,int32_t *Frequency));
    G35DDC_METHOD(int,SetNotchFilterBandwidth,(G35DDC_THIS uint32_t Channel,uint32_t FilterIndex,uint32_t Bandwidth));
    G35DDC_METHOD(int,GetNotchFilterBandwidth,(G35DDC_THIS uint32_t Channel,uint32_t FilterIndex,uint32_t *Bandwidth));
    G35DDC_METHOD(int,SetNotchFilterLength,(G35DDC_THIS uint32_t Channel,uint32_t FilterIndex,uint32_t Length));
    G35DDC_METHOD(int,GetNotchFilterLength,(G35DDC_THIS uint32_t Channel,uint32_t FilterIndex,uint32_t *Length));

    G35DDC_METHOD(int,SetDemodulatorMode,(G35DDC_THIS uint32_t Channel,uint32_t Mode));
    G35DDC_METHOD(int,GetDemodulatorMode,(G35DDC_THIS uint32_t Channel,uint32_t *Mode));
    G35DDC_METHOD(int,SetDemodulatorFrequency,(G35DDC_THIS uint32_t Channel,int32_t Frequency));
    G35DDC_METHOD(int,GetDemodulatorFrequency,(G35DDC_THIS uint32_t Channel,int32_t *Frequency));
    G35DDC_METHOD(int,SetDemodulatorFilterBandwidth,(G35DDC_THIS uint32_t Channel,uint32_t Bandwidth));
    G35DDC_METHOD(int,GetDemodulatorFilterBandwidth,(G35DDC_THIS uint32_t Channel,uint32_t *Bandwidth));
    G35DDC_METHOD(int,SetDemodulatorFilterShift,(G35DDC_THIS uint32_t Channel,int32_t Shift));
    G35DDC_METHOD(int,GetDemodulatorFilterShift,(G35DDC_THIS uint32_t Channel,int32_t *Shift));
    G35DDC_METHOD(int,SetDemodulatorFilterLength,(G35DDC_THIS uint32_t Channel,uint32_t Length));
    G35DDC_METHOD(int,GetDemodulatorFilterLength,(G35DDC_THIS uint32_t Channel,uint32_t *Length));

    G35DDC_METHOD(int,SetDemodulatorParam,(G35DDC_THIS uint32_t Channel,uint32_t Code,const void *Buffer,uint32_t BufferSize));
    G35DDC_METHOD(int,GetDemodulatorParam,(G35DDC_THIS uint32_t Channel,uint32_t Code,void *Buffer,uint32_t BufferSize));
    G35DDC_METHOD(int,GetDemodulatorState,(G35DDC_THIS uint32_t Channel,uint32_t Code,void *Buffer,uint32_t BufferSize));

    G35DDC_METHOD(int,StartAudio,(G35DDC_THIS uint32_t Channel,uint32_t SamplesPerBuffer));
    G35DDC_METHOD(int,StopAudio,(G35DDC_THIS uint32_t Channel));

    G35DDC_METHOD(int,StartAudioPlayback,(G35DDC_THIS uint32_t Channel,uint32_t SamplesPerBuffer));
    G35DDC_METHOD(int,PauseAudioPlayback,(G35DDC_THIS uint32_t Channel));
    G35DDC_METHOD(int,ResumeAudioPlayback,(G35DDC_THIS uint32_t Channel));

    G35DDC_METHOD(int,SetAudioFilter,(G35DDC_THIS uint32_t Channel,int Enabled));
    G35DDC_METHOD(int,GetAudioFilter,(G35DDC_THIS uint32_t Channel,int *Enabled));
    G35DDC_METHOD(int,SetAudioFilterParams,(G35DDC_THIS uint32_t Channel,uint32_t CutOffLow,uint32_t CutOffHigh,double Deemphasis));
    G35DDC_METHOD(int,GetAudioFilterParams,(G35DDC_THIS uint32_t Channel,uint32_t *CutOffLow,uint32_t *CutOffHigh,double *Deemphasis));
    G35DDC_METHOD(int,SetAudioFilterLength,(G35DDC_THIS uint32_t Channel,uint32_t Length));
    G35DDC_METHOD(int,GetAudioFilterLength,(G35DDC_THIS uint32_t Channel,uint32_t *Length));
    G35DDC_METHOD(int,SetAudioGain,(G35DDC_THIS uint32_t Channel,double Gain));
    G35DDC_METHOD(int,GetAudioGain,(G35DDC_THIS uint32_t Channel,double *Gain));
    G35DDC_METHOD(int,SetVolume,(G35DDC_THIS uint32_t Channel,uint8_t Volume));
    G35DDC_METHOD(int,GetVolume,(G35DDC_THIS uint32_t Channel,uint8_t *Volume));
    G35DDC_METHOD(int,SetMute,(G35DDC_THIS uint32_t Channel,int Mute));
    G35DDC_METHOD(int,GetMute,(G35DDC_THIS uint32_t Channel,int *Mute));

    G35DDC_METHOD(int,GetSpectrumCompensation,(G35DDC_THIS uint32_t CenterFrequency,uint32_t Bandwidth,float *Buffer,uint32_t Count));

    G35DDC_METHOD(int,SetFrequency,(G35DDC_THIS uint32_t Channel,uint32_t Frequency));
    G35DDC_METHOD(int,GetFrequency,(G35DDC_THIS uint32_t Channel,uint32_t *Frequency));

    G35DDC_METHOD(int,GetTemperature,(G35DDC_THIS uint32_t *Temperature));
    G35DDC_METHOD(int,GetDeviceState,(G35DDC_THIS uint32_t *State));

    G35DDC_METHOD(int,SetDAC,(G35DDC_THIS uint32_t DAC));
    G35DDC_METHOD(int,GetDAC,(G35DDC_THIS uint32_t *DAC));

    G35DDC_METHOD(int,SetBuiltInTest,(G35DDC_THIS int Enabled));
    G35DDC_METHOD(int,GetBuiltInTest,(G35DDC_THIS int *Enabled));
};


#undef G35DDC_INTERFACE
#define G35DDC_INTERFACE ICohG35DDCDeviceSetEnumerator

G35DDC_DECLARE_INTERFACE(ICohG35DDCDeviceSetEnumerator)
{
    G35DDC_METHOD_VOID(uint32_t,AddRef);
    G35DDC_METHOD_VOID(uint32_t,Release);

    G35DDC_METHOD_VOID(int,Enumerate);
    G35DDC_METHOD_VOID(uint32_t,GetDeviceSetCount);
    G35DDC_METHOD(int,GetDeviceSetInfo,(G35DDC_THIS uint32_t SetIndex,G35DDC_DEVICE_INFO *DeviceInfos,uint32_t *DeviceInfosCount));
};



#undef INTERFACE
#define INTERFACE ICohG35DDCDeviceSetCallback

struct ICohG35DDCDeviceSet;

G35DDC_DECLARE_INTERFACE(ICohG35DDCDeviceSetCallback)
{
    G35DDC_METHOD(void,CohG35DDC_DDC1StreamCallback,(G35DDC_THIS struct ICohG35DDCDeviceSet *DeviceSet,uint32_t DeviceCount,const void **Buffers,uint32_t NumberOfSamples,uint32_t BitsPerSample));
    G35DDC_METHOD(void,CohG35DDC_IFCallback,(G35DDC_THIS struct ICohG35DDCDeviceSet *DeviceSet,uint32_t DeviceIndex,const int16_t *Buffer,uint32_t NumberOfSamples,uint16_t MaxADCAmplitude,uint32_t ADCSamplingRate));
    G35DDC_METHOD(void,CohG35DDC_DDC2StreamCallback,(G35DDC_THIS struct ICohG35DDCDeviceSet *DeviceSet,uint32_t DeviceIndex,const float *Buffer,uint32_t NumberOfSamples));
    //SlevelPeak and SlevelRMS are in Volts
    //to convert SlevelRMS to power in dBm use SlevelRMS_dBm=10.0*log10(SlevelRMS*SlevelRMS/50.0*1000.0); where 50.0 is receiver input impendance, 1000.0 is coefficents to convert Watts to mili Watts
    G35DDC_METHOD(void,CohG35DDC_DDC2PreprocessedStreamCallback,(G35DDC_THIS struct ICohG35DDCDeviceSet *Device,uint32_t DeviceIndex,const float *Buffer,uint32_t NumberOfSamples,float SlevelPeak,float SlevelRMS));
    G35DDC_METHOD(void,CohG35DDC_AudioStreamCallback,(G35DDC_THIS struct ICohG35DDCDeviceSet *Device,uint32_t DeviceIndex,uint32_t Type,const float *Buffer,uint32_t NumberOfSamples));
};

#undef INTERFACE
#define INTERFACE ICohG35DDCDeviceSet

G35DDC_DECLARE_INTERFACE(ICohG35DDCDeviceSet)
{
    G35DDC_METHOD_VOID(uint32_t,AddRef);
    G35DDC_METHOD_VOID(uint32_t,Release);

    G35DDC_METHOD(int,Open,(G35DDC_THIS G35DDC_DEVICE_INFO *DeviceInfos,uint32_t Count));
    G35DDC_METHOD_VOID(void,Close);
    G35DDC_METHOD_VOID(int,IsOpen);

    G35DDC_METHOD(int,GetDeviceCount,(G35DDC_THIS uint32_t *Count));

    G35DDC_METHOD_VOID(int,IsConnected);

    G35DDC_METHOD(void,SetCallback,(G35DDC_THIS ICohG35DDCDeviceSetCallback *Callback));
    G35DDC_METHOD_VOID(ICohG35DDCDeviceSetCallback*,GetCallback);

    G35DDC_METHOD(int,GetDeviceInfo,(G35DDC_THIS uint32_t DeviceIndex,G35DDC_DEVICE_INFO *Info));
    G35DDC_METHOD(int,GetDeviceInfos,(G35DDC_THIS G35DDC_DEVICE_INFO *Infos,uint32_t *InfoCount));

    G35DDC_METHOD(int,SetPower,(G35DDC_THIS int Power));
    G35DDC_METHOD(int,GetPower,(G35DDC_THIS int *Power));

    G35DDC_METHOD(int,SetAttenuator,(G35DDC_THIS uint32_t Attenuator));
    G35DDC_METHOD(int,GetAttenuator,(G35DDC_THIS uint32_t *Attenuator));

    G35DDC_METHOD(int,SetPreselectors,(G35DDC_THIS uint32_t Low,uint32_t High));
    G35DDC_METHOD(int,GetPreselectors,(G35DDC_THIS uint32_t *Low,uint32_t *High));

    G35DDC_METHOD(int,SetPreamplifier,(G35DDC_THIS int Preamp));
    G35DDC_METHOD(int,GetPreamplifier,(G35DDC_THIS int *Preamp));

    G35DDC_METHOD(int,SetInverted,(G35DDC_THIS int Inverted));
    G35DDC_METHOD(int,GetInverted,(G35DDC_THIS int *Inverted));

    G35DDC_METHOD(int,SetADCNoiseBlanker,(G35DDC_THIS int Enabled));
    G35DDC_METHOD(int,GetADCNoiseBlanker,(G35DDC_THIS int *Enabled));
    G35DDC_METHOD(int,SetADCNoiseBlankerThreshold,(G35DDC_THIS uint16_t Threshold));
    G35DDC_METHOD(int,GetADCNoiseBlankerThreshold,(G35DDC_THIS uint16_t *Threshold));

    G35DDC_METHOD(int,StartIF,(G35DDC_THIS uint32_t DeviceIndex,uint16_t Period));
    G35DDC_METHOD(int,StopIF,(G35DDC_THIS uint32_t DeviceIndex));

    G35DDC_METHOD(int,GetDDCInfo,(G35DDC_THIS uint32_t DDCTypeIndex,G3XDDC_DDC_INFO *Info));
    G35DDC_METHOD(int,GetDDC1Count,(G35DDC_THIS uint32_t *Count));
    G35DDC_METHOD(int,SetDDC1,(G35DDC_THIS uint32_t DDCTypeIndex));
    G35DDC_METHOD(int,GetDDC1,(G35DDC_THIS uint32_t *DDCTypeIndex,G3XDDC_DDC_INFO *DDCInfo));
    G35DDC_METHOD(int,StartDDC1,(G35DDC_THIS uint32_t SamplesPerBuffer));
    G35DDC_METHOD_VOID(int,StopDDC1);
    G35DDC_METHOD(int,SetDDC1Frequency,(G35DDC_THIS uint32_t Frequency));
    G35DDC_METHOD(int,GetDDC1Frequency,(G35DDC_THIS uint32_t *Frequency));
    G35DDC_METHOD(int,SetDDC1PhaseShift,(G35DDC_THIS uint32_t DeviceIndex,double PhaseShift));
    G35DDC_METHOD(int,GetDDC1PhaseShift,(G35DDC_THIS uint32_t DeviceIndex,double *PhaseShift));

    G35DDC_METHOD(int,GetDDC2,(G35DDC_THIS uint32_t *DDCTypeIndex,G3XDDC_DDC_INFO *DDCInfo));

    G35DDC_METHOD(int,StartDDC2,(G35DDC_THIS uint32_t DeviceIndex,uint32_t SamplesPerBuffer));
    G35DDC_METHOD(int,StopDDC2,(G35DDC_THIS uint32_t DeviceIndex));

    G35DDC_METHOD(int,SetDDC2Frequency,(G35DDC_THIS uint32_t DeviceIndex,int32_t Frequency));
    G35DDC_METHOD(int,GetDDC2Frequency,(G35DDC_THIS uint32_t DeviceIndex,int32_t *Frequency));

    G35DDC_METHOD(int,SetDDC2NoiseBlanker,(G35DDC_THIS uint32_t DeviceIndex,int Enabled));
    G35DDC_METHOD(int,GetDDC2NoiseBlanker,(G35DDC_THIS uint32_t DeviceIndex,int *Enabled));
    G35DDC_METHOD(int,SetDDC2NoiseBlankerThreshold,(G35DDC_THIS uint32_t DeviceIndex,double Threshold));
    G35DDC_METHOD(int,GetDDC2NoiseBlankerThreshold,(G35DDC_THIS uint32_t DeviceIndex,double *Threshold));
    G35DDC_METHOD(int,GetDDC2NoiseBlankerExcessValue,(G35DDC_THIS uint32_t DeviceIndex,double *Value));

    G35DDC_METHOD(int,GetSignalLevel,(G35DDC_THIS uint32_t DeviceIndex,float *Peak,float *RMS));

    G35DDC_METHOD(int,SetAGC,(G35DDC_THIS uint32_t DeviceIndex,int Enabled));
    G35DDC_METHOD(int,GetAGC,(G35DDC_THIS uint32_t DeviceIndex,int *Enabled));
    G35DDC_METHOD(int,SetAGCParams,(G35DDC_THIS uint32_t DeviceIndex,double AttackTime,double DecayTime,double ReferenceLevel));
    G35DDC_METHOD(int,GetAGCParams,(G35DDC_THIS uint32_t DeviceIndex,double *AttackTime,double *DecayTime,double *ReferenceLevel));
    G35DDC_METHOD(int,SetMaxAGCGain,(G35DDC_THIS uint32_t DeviceIndex,double MaxGain));
    G35DDC_METHOD(int,GetMaxAGCGain,(G35DDC_THIS uint32_t DeviceIndex,double *MaxGain));
    G35DDC_METHOD(int,SetGain,(G35DDC_THIS uint32_t DeviceIndex,double Gain));
    G35DDC_METHOD(int,GetGain,(G35DDC_THIS uint32_t DeviceIndex,double *Gain));
    G35DDC_METHOD(int,GetCurrentGain,(G35DDC_THIS uint32_t DeviceIndex,double *Gain));

    G35DDC_METHOD(int,SetNotchFilter,(G35DDC_THIS uint32_t DeviceIndex,uint32_t NotchFilterIndex,int Enabled));
    G35DDC_METHOD(int,GetNotchFilter,(G35DDC_THIS uint32_t DeviceIndex,uint32_t NotchFilterIndex,int *Enabled));
    G35DDC_METHOD(int,SetNotchFilterFrequency,(G35DDC_THIS uint32_t DeviceIndex,uint32_t NotchFilterIndex,int32_t Frequency));
    G35DDC_METHOD(int,GetNotchFilterFrequency,(G35DDC_THIS uint32_t DeviceIndex,uint32_t NotchFilterIndex,int32_t *Frequency));
    G35DDC_METHOD(int,SetNotchFilterBandwidth,(G35DDC_THIS uint32_t DeviceIndex,uint32_t NotchFilterIndex,uint32_t Bandwidth));
    G35DDC_METHOD(int,GetNotchFilterBandwidth,(G35DDC_THIS uint32_t DeviceIndex,uint32_t NotchFilterIndex,uint32_t *Bandwidth));
    G35DDC_METHOD(int,SetNotchFilterLength,(G35DDC_THIS uint32_t DeviceIndex,uint32_t NotchFilterIndex,uint32_t Length));         //Length: min=64,max=32768,step=64
    G35DDC_METHOD(int,GetNotchFilterLength,(G35DDC_THIS uint32_t DeviceIndex,uint32_t NotchFilterIndex,uint32_t *Length));


    G35DDC_METHOD(int,SetDemodulatorMode,(G35DDC_THIS uint32_t DeviceIndex,uint32_t Mode));
    G35DDC_METHOD(int,GetDemodulatorMode,(G35DDC_THIS uint32_t DeviceIndex,uint32_t *Mode));
    G35DDC_METHOD(int,SetDemodulatorFrequency,(G35DDC_THIS uint32_t DeviceIndex,int32_t Frequency));
    G35DDC_METHOD(int,GetDemodulatorFrequency,(G35DDC_THIS uint32_t DeviceIndex,int32_t *Frequency));
    G35DDC_METHOD(int,SetDemodulatorFilterBandwidth,(G35DDC_THIS uint32_t DeviceIndex,uint32_t Bandwidth));
    G35DDC_METHOD(int,GetDemodulatorFilterBandwidth,(G35DDC_THIS uint32_t DeviceIndex,uint32_t *Bandwidth));
    G35DDC_METHOD(int,SetDemodulatorFilterShift,(G35DDC_THIS uint32_t DeviceIndex,int32_t Shift));
    G35DDC_METHOD(int,GetDemodulatorFilterShift,(G35DDC_THIS uint32_t DeviceIndex,int32_t *Shift));
    G35DDC_METHOD(int,SetDemodulatorFilterLength,(G35DDC_THIS uint32_t DeviceIndex,uint32_t Length));   //Length: min=64,max=32768,step=64
    G35DDC_METHOD(int,GetDemodulatorFilterLength,(G35DDC_THIS uint32_t DeviceIndex,uint32_t *Length));

    G35DDC_METHOD(int,SetDemodulatorParam,(G35DDC_THIS uint32_t DeviceIndex,uint32_t Code,const void *Buffer,uint32_t BufferSize));
    G35DDC_METHOD(int,GetDemodulatorParam,(G35DDC_THIS uint32_t DeviceIndex,uint32_t Code,void *Buffer,uint32_t BufferSize));
    G35DDC_METHOD(int,GetDemodulatorState,(G35DDC_THIS uint32_t DeviceIndex,uint32_t Code,void *Buffer,uint32_t BufferSize));

    G35DDC_METHOD(int,StartAudio,(G35DDC_THIS uint32_t DeviceIndex,uint32_t SamplesPerBuffer));
    G35DDC_METHOD(int,StopAudio,(G35DDC_THIS uint32_t DeviceIndex));

    G35DDC_METHOD(int,SetAudioFilter,(G35DDC_THIS uint32_t DeviceIndex,int Enabled));
    G35DDC_METHOD(int,GetAudioFilter,(G35DDC_THIS uint32_t DeviceIndex,int *Enabled));
    G35DDC_METHOD(int,SetAudioFilterParams,(G35DDC_THIS uint32_t DeviceIndex,uint32_t CutOffLow,uint32_t CutOffHigh,double Deemphasis));
    G35DDC_METHOD(int,GetAudioFilterParams,(G35DDC_THIS uint32_t DeviceIndex,uint32_t *CutOffLow,uint32_t *CutOffHigh,double *Deemphasis));
    G35DDC_METHOD(int,SetAudioFilterLength,(G35DDC_THIS uint32_t DeviceIndex,uint32_t Length));         //Length: min=64,max=32768,step64
    G35DDC_METHOD(int,GetAudioFilterLength,(G35DDC_THIS uint32_t DeviceIndex,uint32_t *Length));
    G35DDC_METHOD(int,SetAudioGain,(G35DDC_THIS uint32_t DeviceIndex,double Gain));
    G35DDC_METHOD(int,GetAudioGain,(G35DDC_THIS uint32_t DeviceIndex,double *Gain));
    G35DDC_METHOD(int,SetVolume,(G35DDC_THIS uint32_t DeviceIndex,uint8_t Volume));
    G35DDC_METHOD(int,GetVolume,(G35DDC_THIS uint32_t DeviceIndex,uint8_t *Volume));
    G35DDC_METHOD(int,SetMute,(G35DDC_THIS uint32_t DeviceIndex,int Mute));
    G35DDC_METHOD(int,GetMute,(G35DDC_THIS uint32_t DeviceIndex,int *Mute));


    G35DDC_METHOD(int,GetSpectrumCompensation,(G35DDC_THIS uint32_t DeviceIndex,uint32_t CenterFrequency,uint32_t Width,float *Buffer,uint32_t Count));

    G35DDC_METHOD(int,GetTemperature,(G35DDC_THIS uint32_t DeviceIndex,uint32_t *Temperature));
    G35DDC_METHOD(int,GetDeviceState,(G35DDC_THIS uint32_t DeviceIndex,uint32_t *State));

    G35DDC_METHOD(int,SetDAC,(G35DDC_THIS uint32_t DeviceIndex,uint32_t DAC));
    G35DDC_METHOD(int,GetDAC,(G35DDC_THIS uint32_t DeviceIndex,uint32_t *DAC));

    G35DDC_METHOD(int,GetTimestampCounters,(G35DDC_THIS double *DDC1SampleCounter,uint64_t *ADCPeriodCounter));

    G35DDC_METHOD(int,SetBuiltInTest,(G35DDC_THIS uint32_t DeviceIndex,int Enabled));
    G35DDC_METHOD(int,GetBuiltInTest,(G35DDC_THIS uint32_t DeviceIndex,int *Enabled));
};


typedef void (*G35DDC_IF_CALLBACK)(const int16_t *Buffer,uint32_t NumberOfSamples,uint16_t MaxADCAmplitude,uint32_t ADCSampleRate,uintptr_t UserData);
typedef void (*G35DDC_DDC1_STREAM_CALLBACK)(const void *Buffer,uint32_t NumberOfSamples,uint32_t BitsPerSample,uintptr_t UserData);
typedef int (*G35DDC_DDC1_PLAYBACK_STREAM_CALLBACK)(void *Buffer,uint32_t NumberOfSamples,uint32_t BitsPerSample,uintptr_t UserData);
typedef void (*G35DDC_DDC2_STREAM_CALLBACK)(uint32_t Channel,const float *Buffer,uint32_t NumberOfSamples,uintptr_t UserData);
typedef void (*G35DDC_DDC2_PREPROCESSED_STREAM_CALLBACK)(uint32_t Channel,const float *Buffer,uint32_t NumberOfSamples,float SlevelPeak,float SlevelRMS,uintptr_t UserData);
typedef void (*G35DDC_AUDIO_STREAM_CALLBACK)(uint32_t Channel,uint32_t Type,const float *Buffer,uint32_t NumberOfSamples,uintptr_t UserData);
typedef int (*G35DDC_AUDIO_PLAYBACK_STREAM_CALLBACK)(uint32_t Channel,float *Buffer,uint32_t NumberOfSamples,uintptr_t UserData);

typedef struct
{
    G35DDC_IF_CALLBACK                          IFCallback;
    G35DDC_DDC1_STREAM_CALLBACK                 DDC1StreamCallback;
    G35DDC_DDC1_PLAYBACK_STREAM_CALLBACK        DDC1PlaybackStreamCallback;
    G35DDC_DDC2_STREAM_CALLBACK                 DDC2StreamCallback;
    G35DDC_DDC2_PREPROCESSED_STREAM_CALLBACK    DDC2PreprocessedStreamCallback;
    G35DDC_AUDIO_STREAM_CALLBACK                AudioStreamCallback;
    G35DDC_AUDIO_PLAYBACK_STREAM_CALLBACK       AudioPlaybackStreamCallback;
} G35DDC_CALLBACKS;

typedef int (*G35DDC_GET_DEVICE_LIST)(G35DDC_DEVICE_INFO *DeviceList,uint32_t BufferSize);
typedef int (*G35DDC_OPEN_DEVICE)(const char *SerialNumber);
typedef int (*G35DDC_CLOSE_DEVICE)(int32_t Handle);
typedef int (*G35DDC_IS_DEVICE_CONNECTED)(int32_t Handle);
typedef int (*G35DDC_GET_DEVICE_INFO)(int32_t Handle,G35DDC_DEVICE_INFO *Info,uint32_t BufferLength);
typedef int (*G35DDC_SET_POWER)(int32_t Handle,int Power);
typedef int (*G35DDC_GET_POWER)(int32_t Handle,int *Power);
typedef int (*G35DDC_SET_EXTERNAL_REFERENCE)(int32_t Handle,int Enabled);
typedef int (*G35DDC_GET_EXTERNAL_REFERENCE)(int32_t Handle,int *Enabled);
typedef int (*G35DDC_SET_ATTENUATOR)(int32_t Handle,uint32_t Attenuator);
typedef int (*G35DDC_GET_ATTENUATOR)(int32_t Handle,uint32_t *Attenuator);
typedef int (*G35DDC_SET_PREAMPLIFIER)(int32_t Handle,int Preamp);
typedef int (*G35DDC_GET_PREAMPLIFIER)(int32_t Handle,int *Preamp);
typedef int (*G35DDC_SET_ADC_NOISE_BLANKER)(int32_t Handle,int Enabled);
typedef int (*G35DDC_GET_ADC_NOISE_BLANKER)(int32_t Handle,int *Enabled);
typedef int (*G35DDC_SET_ADC_NOISE_BLANKER_THRESHOLD)(int32_t Handle,uint16_t Threshold);
typedef int (*G35DDC_GET_ADC_NOISE_BLANKER_THRESHOLD)(int32_t Handle,uint16_t *Threshold);
typedef int (*G35DDC_SET_FREQUENCY)(int32_t Handle,uint32_t Channel,uint64_t Frequency);
typedef int (*G35DDC_GET_FREQUENCY)(int32_t Handle,uint32_t Channel,uint64_t *Frequency);
typedef int (*G35DDC_START_IF)(int32_t Handle,uint16_t Period);
typedef int (*G35DDC_STOP_IF)(int32_t Handle);
typedef int (*G35DDC_GET_DDC_INFO)(uint32_t DDCIndex,G3XDDC_DDC_INFO *Info);
typedef int (*G35DDC_GET_DDC1_COUNT)(int32_t Handle,uint32_t *Count);
typedef int (*G35DDC_SET_DDC1)(int32_t Handle,uint32_t DDCIndex);
typedef int (*G35DDC_GET_DDC1)(int32_t Handle,uint32_t *DDCIndex,G3XDDC_DDC_INFO *DDCInfo);
typedef int (*G35DDC_START_DDC1)(int32_t Handle,uint32_t SamplesPerBuffer);
typedef int (*G35DDC_STOP_DDC1)(int32_t Handle);
typedef int (*G35DDC_SET_DDC1_FREQUENCY)(int32_t Handle,int32_t Frequency);
typedef int (*G35DDC_GET_DDC1_FREQUENCY)(int32_t Handle,int32_t *Frequency);
typedef int (*G35DDC_START_DDC1_PLAYBACK)(int32_t Handle,uint32_t SamplesPerBuffer,uint32_t BitsPerSample);
typedef int (*G35DDC_PAUSE_DDC1_PLAYBACK)(int32_t Handle);
typedef int (*G35DDC_RESUME_DDC1_PLAYBACK)(int32_t Handle);

typedef int (*G35DDC_GET_DDC2)(int32_t Handle,uint32_t *DDCTypeIndex,G3XDDC_DDC_INFO *DDCInfo);
typedef int (*G35DDC_START_DDC2)(int32_t Handle,uint32_t Channel,uint32_t SamplesPerBuffer);
typedef int (*G35DDC_STOP_DDC2)(int32_t Handle,uint32_t Channel);
typedef int (*G35DDC_SET_DDC2_FREQUENCY)(int32_t Handle,uint32_t Channel,int32_t Frequency);
typedef int (*G35DDC_GET_DDC2_FREQUENCY)(int32_t Handle,uint32_t Channel,int32_t *Frequency);

typedef int (*G35DDC_SET_AGC)(int32_t Handle,uint32_t Channel,int Enabled);
typedef int (*G35DDC_GET_AGC)(int32_t Handle,uint32_t Channel,int *Enabled);
typedef int (*G35DDC_SET_AGC_PARAMS)(int32_t Handle,uint32_t Channel,double AttackTime,double DecayTime,double ReferenceLevel);
typedef int (*G35DDC_GET_AGC_PARAMS)(int32_t Handle,uint32_t Channel,double *AttackTime,double *DecayTime,double *ReferenceLevel);
typedef int (*G35DDC_SET_MAX_AGC_GAIN)(int32_t Handle,uint32_t Channel,double MaxGain);
typedef int (*G35DDC_GET_MAX_AGC_GAIN)(int32_t Handle,uint32_t Channel,double *MaxGain);
typedef int (*G35DDC_SET_GAIN)(int32_t Handle,uint32_t Channel,double Gain);
typedef int (*G35DDC_GET_GAIN)(int32_t Handle,uint32_t Channel,double *Gain);
typedef int (G35DDC_GET_CURRENT_GAIN)(int32_t Handle,uint32_t Channel,double *Gain);

typedef int (*G35DDC_SET_NOTCH_FILTER)(int32_t Handle,uint32_t Channel,uint32_t FilterIndex,int Enabled);
typedef int (*G35DDC_GET_NOTCH_FILTER)(int32_t Handle,uint32_t Channel,uint32_t FilterIndex,int *Enabled);
typedef int (*G35DDC_SET_NOTCH_FILTER_FREEQUENCY)(int32_t Handle,uint32_t Channel,uint32_t FilterIndex,int32_t Frequency);
typedef int (*G35DDC_GET_NOTCH_FILTER_FREEQUENCY)(int32_t Handle,uint32_t Channel,uint32_t FilterIndex,int32_t *Frequency);
typedef int (*G35DDC_SET_NOTCH_FILTER_BANDWIDTH)(int32_t Handle,uint32_t Channel,uint32_t FilterIndex,uint32_t Bandwidth);
typedef int (*G35DDC_GET_NOTCH_FILTER_BANDWIDTH)(int32_t Handle,uint32_t Channel,uint32_t FilterIndex,uint32_t *Bandwidth);
typedef int (*G35DDC_SET_NOTCH_FILTER_LENGTH)(int32_t Handle,uint32_t Channel,uint32_t FilterIndex,uint32_t Length);
typedef int (*G35DDC_GET_NOTCH_FILTER_LENGTH)(int32_t Handle,uint32_t Channel,uint32_t FilterIndex,uint32_t *Length);

typedef int (*G35DDC_SET_DEMODULATOR_FREQUENCY)(int32_t Handle,uint32_t Channel,int32_t Frequency);
typedef int (*G35DDC_GET_DEMODULATOR_FREQUENCY)(int32_t Handle,uint32_t Channel,int32_t *Frequency);
typedef int (*G35DDC_SET_DEMODULATOR_FILTER_BANDWIDTH)(int32_t Handle,uint32_t Channel,uint32_t Bandwidth);
typedef int (*G35DDC_GET_DEMODULATOR_FILTER_BANDWIDTH)(int32_t Handle,uint32_t Channel,uint32_t *Bandwidth);
typedef int (*G35DDC_SET_DEMODULATOR_FILTER_SHIFT)(int32_t Handle,uint32_t Channel,int32_t Shift);
typedef int (*G35DDC_GET_DEMODULATOR_FILTER_SHIFT)(int32_t Handle,uint32_t Channel,int32_t *Shift);
typedef int (*G35DDC_SET_DEMODULATOR_FILTER_LENGTH)(int32_t Handle,uint32_t Channel,uint32_t Length);
typedef int (*G35DDC_GET_DEMODULATOR_FILTER_LENGTH)(int32_t Handle,uint32_t Channel,uint32_t *Length);
typedef int (*G35DDC_SET_DEMODULATOR_MODE)(int32_t Handle,uint32_t Channel,uint32_t Mode);
typedef int (*G35DDC_GET_DEMODULATOR_MODE)(int32_t Handle,uint32_t Channel,uint32_t *Mode);
typedef int (*G35DDC_SET_DEMODULATOR_PARAM)(int32_t Handle,uint32_t Channel,uint32_t Code,const void *Buffer,uint32_t BufferSize);
typedef int (*G35DDC_GET_DEMODULATOR_PARAM)(int32_t Handle,uint32_t Channel,uint32_t Code,void *Buffer,uint32_t BufferSize);
typedef int (*G35DDC_GET_DEMODULATOR_STATE)(int32_t Handle,uint32_t Channel,uint32_t Code,void *Buffer,uint32_t BufferSize);

typedef int (*G35DDC_GET_AUDIO_SAMPLE_RATE_COUNT)(int32_t Handle,uint32_t *Count);
typedef int (*G35DDC_GET_AUDIO_SAMPLE_RATE)(int32_t Handle,uint32_t SampleRateIndex,uint32_t *SampleRate);
typedef int (*G35DDC_START_AUDIO)(int32_t Handle,uint32_t Channel,uint32_t SamplesPerBuffer);
typedef int (*G35DDC_STOP_AUDIO)(int32_t Handle,uint32_t Channel);
typedef int (*G35DDC_SET_AUDIO)(int32_t Handle,uint32_t Channel,uint32_t SampleRateIndex);
typedef int (*G35DDC_GET_AUDIO)(int32_t Handle,uint32_t Channel,uint32_t *SampleRateIndex);
typedef int (*G35DDC_START_AUDIO_PLAYBACK)(int32_t Handle,uint32_t Channel,uint32_t SamplesPerBuffer);
typedef int (*G35DDC_PAUSE_AUDIO_PLAYBACK)(int32_t Handle,uint32_t Channel);
typedef int (*G35DDC_RESUME_AUDIO_PLAYBACK)(int32_t Handle,uint32_t Channel);
typedef int (*G35DDC_SET_AUDIO_FILTER)(int32_t Handle,uint32_t Channel,int Enabled);
typedef int (*G35DDC_GET_AUDIO_FILTER)(int32_t Handle,uint32_t Channel,int *Enabled);
typedef int (*G35DDC_SET_AUDIO_FILTER_PARAMS)(int32_t Handle,uint32_t Channel,uint32_t CutOffLow,uint32_t CutOffHigh,double Deemphasis);
typedef int (*G35DDC_GET_AUDIO_FILTER_PARAMS)(int32_t Handle,uint32_t Channel,uint32_t *CutOffLow,uint32_t *CutOffHigh,double *Deemphasis);
typedef int (*G35DDC_SET_AUDIO_FILTER_LENGTH)(int32_t Handle,uint32_t Channel,uint32_t Length);
typedef int (*G35DDC_GET_AUDIO_FILTER_LENGTH)(int32_t Handle,uint32_t Channel,uint32_t *Length);
typedef int (*G35DDC_SET_AUDIO_GAIN)(int32_t Handle,uint32_t Channel,double Gain);
typedef int (*G35DDC_GET_AUDIO_GAIN)(int32_t Handle,uint32_t Channel,double *Gain);
typedef int (*G35DDC_SET_VOLUME)(int32_t Handle,uint32_t Channel,uint8_t Volume);
typedef int (*G35DDC_GET_VOLUME)(int32_t Handle,uint32_t Channel,uint8_t *Volume);
typedef int (*G35DDC_SET_MUTE)(int32_t Handle,uint32_t Channel,int Mute);
typedef int (*G35DDC_GET_MUTE)(int32_t Handle,uint32_t Channel,int *Mute);

typedef int (*G35DDC_GET_SPECTRUM_COMPENSATION)(int32_t Handle,uint32_t CenterFrequency,uint32_t Bandwidth,float *Buffer,uint32_t Count);

typedef int (*G35DDC_SET_DAC)(int32_t Handle,uint32_t DAC);
typedef int (*G35DDC_GET_DAC)(int32_t Handle,uint32_t *DAC);

typedef int (*G35DDC_GET_TEMPERATURE)(int32_t Handle,uint32_t *Temperature);

typedef int (*G35DDC_SET_INVERTED)(int32_t Handle,int Inverted);
typedef int (*G35DDC_GET_INVERTED)(int32_t Handle,int *Inverted);

typedef int (*G35DDC_SET_DDC2_NOISE_BLANKER)(int32_t Handle,uint32_t Channel,int Enabled);
typedef int (*G35DDC_GET_DDC2_NOISE_BLANKER)(int32_t Handle,uint32_t Channel,int *Enabled);
typedef int (*G35DDC_SET_DDC2_NOISE_BLANKER_THRESHOLD)(int32_t Handle,uint32_t Channel,double Threshold);
typedef int (*G35DDC_GET_DDC2_NOISE_BLANKER_THRESHOLD)(int32_t Handle,uint32_t Channel,double *Threshold);
typedef int (*G35DDC_GET_DDC2_NOISE_BLANKER_EXCESS_VALUE)(int32_t Handle,uint32_t Channel,double *Value);

typedef int (*G35DDC_GET_DEVICE_STATE)(int32_t Handle,uint32_t *State);

typedef int (*G35DDC_GET_SIGNAL_LEVEL)(int32_t Handle,uint32_t Channel,float *Peak,float *RMS);

typedef int (*G35DDC_SET_CALLBACKS)(int32_t Handle,const G35DDC_CALLBACKS *Callbacks,uintptr_t UserData);


typedef int (*G35DDC_SET_BUILT_IN_TEST)(int32_t hDevice,int Enabled);
typedef int (*G35DDC_GET_BUILT_IN_TEST)(int32_t hDevice,int *Enabled);

typedef void (*COH_G35DDC_DDC1_STREAM_CALLBACK)(uint32_t DeviceCount,const void **Buffers,uint32_t NumberOfSamples,uint32_t BitsPerSample,uintptr_t UserData);
typedef void (*COH_G35DDC_IF_CALLBACK)(uint32_t DeviceIndex,const int16_t *Buffer,uint32_t NumberOfSamples,uint16_t MaxADCAmplitude,uint32_t ADCSamplingRate,uintptr_t UserData);
typedef void (*COH_G35DDC_DDC2_STREAM_CALLBACK)(uint32_t DeviceIndex,const float *Buffer,uint32_t NumberOfSamples,uintptr_t UserData);
typedef void (*COH_G35DDC_DDC2_PREPROCESSED_STREAM_CALLBACK)(uint32_t DeviceIndex,const float *Buffer,uint32_t NumberOfSamples,float SlevelPeak,float SlevelRMS,uintptr_t UserData);
typedef void (*COH_G35DDC_AUDIO_STREAM_CALLBACK)(uint32_t DeviceIndex,uint32_t Type,const float *Buffer,uint32_t NumberOfSamples,uintptr_t UserData);


typedef struct
{
    COH_G35DDC_DDC1_STREAM_CALLBACK                 DDC1StreamCallback;
    COH_G35DDC_IF_CALLBACK                          IFCallback;
    COH_G35DDC_DDC2_STREAM_CALLBACK                 DDC2StreamCallback;
    COH_G35DDC_DDC2_PREPROCESSED_STREAM_CALLBACK    DDC2PreprocessedStreamCallback;
    COH_G35DDC_AUDIO_STREAM_CALLBACK                AudioStreamCallback;
} COH_G35DDC_CALLBACKS;

typedef struct
{
    uint32_t            DeviceCount;
    G35DDC_DEVICE_INFO  *DeviceInfo;
} COH_G35DDC_DEVICE_SET;

typedef struct
{
    uint32_t                DeviceSetCount;
    COH_G35DDC_DEVICE_SET   *DeviceSet;
} COH_G35DDC_DEVICE_SET_LIST;



//CohGetDeviceSetList
typedef COH_G35DDC_DEVICE_SET_LIST* (*COH_G35DDC_GET_DEVICE_SET_LIST)(void);
//CohFreeDeviceSetList
typedef void (*COH_G35DDC_FREE_DEVICE_SET_LIST)(COH_G35DDC_DEVICE_SET_LIST *DeviceSetList);
//CohOpenDeviceSet
typedef int32_t (*COH_G35DDC_OPEN_DEVICE_SET)(G35DDC_DEVICE_INFO *DeviceInfos,uint32_t Count);
//CohCloseDeviceSet
typedef int (*COH_G35DDC_CLOSE_DEVICE_SET)(int32_t hDeviceSet);
//CohIsConnected
typedef int (*COH_G35DDC_IS_CONNECTED)(int32_t hDeviceSet);
//CohSetCallbacks
typedef int (*COH_G35DDC_SET_CALLBACKS)(int32_t hDeviceSet,const COH_G35DDC_CALLBACKS *Callbacks,uintptr_t UserData);
//CohGetDeviceCount
typedef int (*COH_G35DDC_GET_DEVICE_COUNT)(int32_t hDeviceSet,uint32_t *Count);
//CohGetDeviceInfo
typedef int (*COH_G35DDC_GET_DEVICE_INFO)(int32_t hDeviceSet,uint32_t DeviceIndex,G35DDC_DEVICE_INFO *Info);
//CohGetDeviceInfos
typedef int (*COH_G35DDC_GET_DEVICE_INFOS)(int32_t hDeviceSet,G35DDC_DEVICE_INFO *Infos,uint32_t *InfoCount);
//CohSetPower
typedef int (*COH_G35DDC_SET_POWER)(int32_t hDeviceSet,int Power);
//CohGetPower
typedef int (*COH_G35DDC_GET_POWER)(int32_t hDeviceSet,int *Power);
//CohSetAttenuator
typedef int (*COH_G35DDC_SET_ATTENUATOR)(int32_t hDeviceSet,uint32_t Attenuator);
//CohGetAttenuator
typedef int (*COH_G35DDC_GET_ATTENUATOR)(int32_t hDeviceSet,uint32_t *Attenuator);
//CohSetPreselectors
typedef int (*COH_G35DDC_SET_PRESELECTORS)(int32_t hDeviceSet,uint32_t Low,uint32_t High);
//CohGetPreselectors
typedef int (*COH_G35DDC_GET_PRESELECTORS)(int32_t hDeviceSet,uint32_t *Low,uint32_t *High);
//CohSetPreamp
typedef int (*COH_G35DDC_SET_PREAMP)(int32_t hDeviceSet,int Preamp);
//CohGetPreamp
typedef int (*COH_G35DDC_GET_PREAMP)(int32_t hDeviceSet,int *Preamp);
//CohSetInverted
typedef int (*COH_G35DDC_SET_INVERTED)(int32_t hDeviceSet,int Inverted);
//CohGetInverted
typedef int (*COH_G35DDC_GET_INVERTED)(int32_t hDeviceSet,int *Inverted);
//CohSetADCNoiseBlanker
typedef int (*COH_G35DDC_SET_ADC_NOISE_BLANKER)(int32_t hDeviceSet,int Enabled);
//CohGetADCNoiseBlanker
typedef int (*COH_G35DDC_GET_ADC_NOISE_BLANKER)(int32_t hDeviceSet,int *Enabled);
//CohSetADCNoiseBlankerThreshold
typedef int (*COH_G35DDC_SET_ADC_NOISE_BLANKER_THRESHOLD)(int32_t hDeviceSet,uint16_t Threshold);
//CohGetADCNoiseBlankerThreshold
typedef int (*COH_G35DDC_GET_ADC_NOISE_BLANKER_THRESHOLD)(int32_t hDeviceSet,uint16_t *Threshold);
//CohStartIF
typedef int (*COH_G35DDC_START_IF)(int32_t hDeviceSet,uint32_t DeviceIndex,uint16_t Period);
//CohStopIF
typedef int (*COH_G35DDC_STOP_IF)(int32_t hDeviceSet,uint32_t DeviceIndex);
//CohGetDDCInfo
typedef int (*COH_G35DDC_GET_DDC_INFO)(int32_t hDeviceSet,uint32_t DDCTypeIndex,G3XDDC_DDC_INFO *Info);
//CohGetDDC1Count
typedef int (*COH_G35DDC_GET_DDC1_COUNT)(int32_t hDeviceSet,uint32_t *Count);
//CohSetDDC1
typedef int (*COH_G35DDC_SET_DDC1)(int32_t hDeviceSet,uint32_t DDCTypeIndex);
//CohGetDDC1
typedef int (*COH_G35DDC_GET_DDC1)(int32_t hDeviceSet,uint32_t *DDCTypeIndex,G3XDDC_DDC_INFO *DDCInfo);
//CohStartDDC1
typedef int (*COH_G35DDC_START_DDC1)(int32_t hDeviceSet,uint32_t SamplesPerBuffer);
//CohStopDDC1
typedef int (*COH_G35DDC_STOP_DDC1)(int32_t hDeviceSet);
//CohSetDDC1Frequency
typedef int (*COH_G35DDC_SET_DDC1_FREQUENCY)(int32_t hDeviceSet,uint32_t Frequency);
//CohGetDDC1Frequency
typedef int (*COH_G35DDC_GET_DDC1_FREQUENCY)(int32_t hDeviceSet,uint32_t *Frequency);
//CohSetDDC1PhaseShift
typedef int (*COH_G35DDC_SET_DDC1_PHASE_SHIFT)(int32_t hDeviceSet,uint32_t DeviceIndex,double PhaseShift);
//CohGetDDC1PhaseShift
typedef int (*COH_G35DDC_GET_DDC1_PHASE_SHIFT)(int32_t hDeviceSet,uint32_t DeviceIndex,double *PhaseShift);
//CohGetDDC2
typedef int (*COH_G35DDC_GET_DDC2)(int32_t hDeviceSet,uint32_t *DDCTypeIndex,G3XDDC_DDC_INFO *DDCInfo);
//CohStartDDC2
typedef int (*COH_G35DDC_START_DDC2)(int32_t hDeviceSet,uint32_t DeviceIndex,uint32_t SamplesPerBuffer);
//CohStopDDC2
typedef int (*COH_G35DDC_STOP_DDC2)(int32_t hDeviceSet,uint32_t DeviceIndex);
//CohSetDDC2Frequency
typedef int (*COH_G35DDC_SET_DDC2_FREQUENCY)(int32_t hDeviceSet,uint32_t DeviceIndex,int32_t Frequency);
//CohGetDDC2Frequency
typedef int (*COH_G35DDC_GET_DDC2_FREQUENCY)(int32_t hDeviceSet,uint32_t DeviceIndex,int32_t *Frequency);
//CohSetDDC2NoiseBlanker
typedef int (*COH_G35DDC_SET_DDC2_NOISE_BLANKER)(int32_t hDeviceSet,uint32_t DeviceIndex,int Enabled);
//CohGetDDC2NoiseBlanker
typedef int (*COH_G35DDC_GET_DDC2_NOISE_BLANKER)(int32_t hDeviceSet,uint32_t DeviceIndex,int *Enabled);
//CohSetDDC2NoiseBlankerThreshold
typedef int (*COH_G35DDC_SET_DDC2_NOISE_BLANKER_THRESHOLD)(int32_t hDeviceSet,uint32_t DeviceIndex,double Threshold);
//CohGetDDC2NoiseBlankerThreshold
typedef int (*COH_G35DDC_GET_DDC2_NOISE_BLANKER_THRESHOLD)(int32_t hDeviceSet,uint32_t DeviceIndex,double *Threshold);
//CohGetDDC2NoiseBlankerExcessValue
typedef int (*COH_G35DDC_GET_DDC2_NOISE_BLANKER_EXCESS_VALUE)(int32_t hDeviceSet,uint32_t DeviceIndex,double *Value);
//CohGetSignalLevel
typedef int (*COH_G35DDC_GET_SIGNAL_LEVEL)(int32_t hDeviceSet,uint32_t DeviceIndex,float *Peak,float *RMS);
//CohSetAGC
typedef int (*COH_G35DDC_SET_AGC)(int32_t hDeviceSet,uint32_t DeviceIndex,int Enabled);
//CohGetAGC
typedef int (*COH_G35DDC_GET_AGC)(int32_t hDeviceSet,uint32_t DeviceIndex,int *Enabled);
//CohSetAGCParams
typedef int (*COH_G35DDC_SET_AGC_PARAMS)(int32_t hDeviceSet,uint32_t DeviceIndex,double AttackTime,double DecayTime,double ReferenceLevel);
//CohGetAGCParams
typedef int (*COH_G35DDC_GET_AGC_PARAMS)(int32_t hDeviceSet,uint32_t DeviceIndex,double *AttackTime,double *DecayTime,double *ReferenceLevel);
//CohSetMaxAGCGain
typedef int (*COH_G35DDC_SET_MAX_AGC_GAIN)(int32_t hDeviceSet,uint32_t DeviceIndex,double MaxGain);
//CohGetMaxAGCGain
typedef int (*COH_G35DDC_GET_MAX_AGC_GAIN)(int32_t hDeviceSet,uint32_t DeviceIndex,double *MaxGain);
//CohSetGain
typedef int (*COH_G35DDC_SET_GAIN)(int32_t hDeviceSet,uint32_t DeviceIndex,double Gain);
//CohGetGain
typedef int (*COH_G35DDC_GET_GAIN)(int32_t hDeviceSet,uint32_t DeviceIndex,double *Gain);
//CohGetCurrentGain
typedef int (*COH_G35DDC_GET_CURRENT_GAIN)(int32_t hDeviceSet,uint32_t DeviceIndex,double *Gain);
//CohSetNotchFilter
typedef int (*COH_G35DDC_SET_NOTCH_FILTER)(int32_t hDeviceSet,uint32_t DeviceIndex,uint32_t NotchFilterIndex,int Enabled);
//CohGetNotchFilter
typedef int (*COH_G35DDC_GET_NOTCH_FILTER)(int32_t hDeviceSet,uint32_t DeviceIndex,uint32_t NotchFilterIndex,int *Enabled);
//CohSetNotchFilterFrequency
typedef int (*COH_G35DDC_SET_NOTCH_FILTER_FREQUENCY)(int32_t hDeviceSet,uint32_t DeviceIndex,uint32_t NotchFilterIndex,int32_t Frequency);
//CohGetNotchFilterFrequency
typedef int (*COH_G35DDC_GET_NOTCH_FILTER_FREQUENCY)(int32_t hDeviceSet,uint32_t DeviceIndex,uint32_t NotchFilterIndex,int32_t *Frequency);
//CohSetNotchFilterBandwidth
typedef int (*COH_G35DDC_SET_NOTCH_FILTER_BANDWIDTH)(int32_t hDeviceSet,uint32_t DeviceIndex,uint32_t NotchFilterIndex,uint32_t Bandwidth);
//CohGetNotchFilterBandwidth
typedef int (*COH_G35DDC_GET_NOTCH_FILTER_BANDWIDTH)(int32_t hDeviceSet,uint32_t DeviceIndex,uint32_t NotchFilterIndex,uint32_t *Bandwidth);
//CohSetNotchFilterLength
typedef int (*COH_G35DDC_SET_NOTCH_FILTER_LENGTH)(int32_t hDeviceSet,uint32_t DeviceIndex,uint32_t NotchFilterIndex,uint32_t Length);
//CohGetNotchFilterLength
typedef int (*COH_G35DDC_GET_NOTCH_FILTER_LENGTH)(int32_t hDeviceSet,uint32_t DeviceIndex,uint32_t NotchFilterIndex,uint32_t *Length);
//CohSetDemodulatorMode
typedef int (*COH_G35DDC_SET_DEMODULATOR_MODE)(int32_t hDeviceSet,uint32_t DeviceIndex,uint32_t Mode);
//CohGetDemodulatorMode
typedef int (*COH_G35DDC_GET_DEMODULATOR_MODE)(int32_t hDeviceSet,uint32_t DeviceIndex,uint32_t *Mode);
//CohSetDemodulatorFrequency
typedef int (*COH_G35DDC_SET_DEMODULATOR_FREQUENCY)(int32_t hDeviceSet,uint32_t DeviceIndex,int32_t Frequency);
//CohGetDemodulatorFrequency
typedef int (*COH_G35DDC_GET_DEMODULATOR_FREQUENCY)(int32_t hDeviceSet,uint32_t DeviceIndex,int32_t *Frequency);
//CohSetDemodulatorFilterBandwidth
typedef int (*COH_G35DDC_SET_DEMODULATOR_FILTER_BANDWIDTH)(int32_t hDeviceSet,uint32_t DeviceIndex,uint32_t Bandwidth);
//CohGetDemodulatorFilterBandwidth
typedef int (*COH_G35DDC_GET_DEMODULATOR_FILTER_BANDWIDTH)(int32_t hDeviceSet,uint32_t DeviceIndex,uint32_t *Bandwidth);
//CohSetDemodulatorFilterShift
typedef int (*COH_G35DDC_SET_DEMODULATOR_FILTER_SHIFT)(int32_t hDeviceSet,uint32_t DeviceIndex,int32_t Shift);
//CohGetDemodulatorFilterShift
typedef int (*COH_G35DDC_GET_DEMODULATOR_FILTER_SHIFT)(int32_t hDeviceSet,uint32_t DeviceIndex,int32_t *Shift);
//CohSetDemodulatorFilterLength
typedef int (*COH_G35DDC_SET_DEMODULATOR_FILTER_LENGTH)(int32_t hDeviceSet,uint32_t DeviceIndex,uint32_t Length);
//CohGetDemodulatorFilterLength
typedef int (*COH_G35DDC_GET_DEMODULATOR_FILTER_LENGTH)(int32_t hDeviceSet,uint32_t DeviceIndex,uint32_t *Length);
//CohSetDemodulatorParam
typedef int (*COH_G35DDC_SET_DEMODULATOR_PARAM)(int32_t hDeviceSet,uint32_t DeviceIndex,uint32_t Code,const void *Buffer,uint32_t BufferSize);
//CohGetDemodulatorParam
typedef int (*COH_G35DDC_GET_DEMODULATOR_PARAM)(int32_t hDeviceSet,uint32_t DeviceIndex,uint32_t Code,void *Buffer,uint32_t BufferSize);
//CohGetDemodulatorState
typedef int (*COH_G35DDC_GET_DEMODULATOR_STATE)(int32_t hDeviceSet,uint32_t DeviceIndex,uint32_t Code,void *Buffer,uint32_t BufferSize);
//CohStartAudio
typedef int (*COH_G35DDC_START_AUDIO)(int32_t hDeviceSet,uint32_t DeviceIndex,uint32_t SamplesPerBuffer);
//CohStopAudio
typedef int (*COH_G35DDC_STOP_AUDIO)(int32_t hDeviceSet,uint32_t DeviceIndex);
//CohSetAudioFilter
typedef int (*COH_G35DDC_SET_AUDIO_FILTER)(int32_t hDeviceSet,uint32_t DeviceIndex,int Enabled);
//CohGetAudioFilter
typedef int (*COH_G35DDC_GET_AUDIO_FILTER)(int32_t hDeviceSet,uint32_t DeviceIndex,int *Enabled);
//CohSetAudioFilterParams
typedef int (*COH_G35DDC_SET_AUDIO_FILTER_PARAMS)(int32_t hDeviceSet,uint32_t DeviceIndex,uint32_t CutOffLow,uint32_t CutOffHigh,double Deemphasis);
//CohGetAudioFilterParams
typedef int (*COH_G35DDC_GET_AUDIO_FILTER_PARAMS)(int32_t hDeviceSet,uint32_t DeviceIndex,uint32_t *CutOffLow,uint32_t *CutOffHigh,double *Deemphasis);
//CohSetAudioFilterLength
typedef int (*COH_G35DDC_SET_AUDIO_FILTER_LENGTH)(int32_t hDeviceSet,uint32_t DeviceIndex,uint32_t Length);
//CohGetAudioFilterLength
typedef int (*COH_G35DDC_GET_AUDIO_FILTER_LENGTH)(int32_t hDeviceSet,uint32_t DeviceIndex,uint32_t *Length);
//CohSetAudioGain
typedef int (*COH_G35DDC_SET_AUDIO_GAIN)(int32_t hDeviceSet,uint32_t DeviceIndex,double Gain);
//CohGetAudioGain
typedef int (*COH_G35DDC_GET_AUDIO_GAIN)(int32_t hDeviceSet,uint32_t DeviceIndex,double *Gain);
//CohSetVolume
typedef int (*COH_G35DDC_SET_VOLUME)(int32_t hDeviceSet,uint32_t DeviceIndex,uint8_t Volume);
//CohGetVolume
typedef int (*COH_G35DDC_GET_VOLUME)(int32_t hDeviceSet,uint32_t DeviceIndex,uint8_t *Volume);
//CohSetMute
typedef int (*COH_G35DDC_SET_MUTE)(int32_t hDeviceSet,uint32_t DeviceIndex,int Mute);
//CohGetMute
typedef int (*COH_G35DDC_GET_MUTE)(int32_t hDeviceSet,uint32_t DeviceIndex,int *Mute);
//CohGetSpectrumCompensation
typedef int (*COH_G35DDC_GET_SPECTRUM_COMPENSATION)(int32_t hDeviceSet,uint32_t DeviceIndex,uint32_t CenterFrequency,uint32_t Width,float *Buffer,uint32_t Count);
//CohGetTemperature
typedef int (*COH_G35DDC_GET_TEMPERATURE)(int32_t hDeviceSet,uint32_t DeviceIndex,uint32_t *Temperature);
//CohGetDeviceState
typedef int (*COH_G35DDC_GET_DEVICE_STATE)(int32_t hDeviceSet,uint32_t DeviceIndex,uint32_t *State);
//CohSetDAC
typedef int (*COH_G35DDC_SET_DAC)(int32_t hDeviceSet,uint32_t DeviceIndex,uint32_t DAC);
//CohGetDAC
typedef int (*COH_G35DDC_GET_DAC)(int32_t hDeviceSet,uint32_t DeviceIndex,uint32_t *DAC);
//CohGetTimestampCounters
typedef int (*COH_G35DDC_GET_TIMESTAMP_COUNTERS)(int32_t hDeviceSet,double *DDC1SampleCounter,uint64_t *ADCPeriodCounter);
//CohSetBuiltInTest
typedef int (*COH_G35DDC_SET_BUILT_IN_TEST)(int32_t hDevice,uint32_t DeviceIndex,int Enabled);
//CohGetBuiltInTest
typedef int (*COH_G35DDC_GET_BUILT_IN_TEST)(int32_t hDevice,uint32_t DeviceIndex,int *Enabled);

#pragma pack(pop)

#endif // __G35DDCAPI_H__
